/*
 * Bundesliga is Copyright (c) 1997-2003 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#define ALLOW_ACCESS_TO_INTERNALS_OF_CONTROLS 1
#include <PalmOS.h>
#include <DateGlue.h>

#include "bl.h"
#include "blrcp.h"

Int16 matchDay, DBNo, matchSymbol;
UInt32 DateOfMatchday;
TeamResultType *TeamResult, *BackupTeamResult;
MemHandle TeamResultHandle, BackupTeamResultHandle;
BlDatabasePreferencesType prefs;
BlApplicationPreferencesType appPrefs;
Boolean deviceCanBeam;
SelectedDBInfo info;
Char* progDatePtr;

/*
 * array that holds pointers to the fields in the main form. t1 and t2 are
 * used to show the encounter, r1 and r2 hold the result. team1 and team2
 * store the teams's index.
 */
struct {
  FieldType *t1, *t2, *r1, *r2;
  Int8 team1, team2;
  UInt32 Date;
  UInt16 penalty;
} match[12];

static TablePtr gotoTable;

statsType stats;

Int16 TableRememberPosFirst = 1;
Int16 ReturnToForm;
static Boolean showDay = true;

static void InitFormBl(void) TBLFUNC;
static void SetDateLabel(Int8 n) TBLFUNC;
static void SetResultHandle(FieldType* fptr, UInt8 n1, UInt8 n2) TBLFUNC;
static void SaveMatchdayResults(void) TBLFUNC;
static void InitFormGoto(void) TBLFUNC;
static void RedrawForm(FormType*) TBLFUNC;
static void DrawIntAt(Int16 v, Coord x, Coord y) TBLFUNC;
static void GetGamesString(Char* buf, Int16 n) TBLFUNC;

/*
 * Redraw table pointed to by "tblP";
 * Arguments:
 * - tblP:     pointer to the table
 * - first:    number of the first row to be shown
 * - rows:     total number of rows in the table
 */
void RedrawTable(TablePtr tblP, 
		 ScrollBarPtr sclP, 
		 Int16 first, 
		 Int16 rows, 
		 Int16 start,
		 Int16 size)
{
  Int16 row, last = first + size - 1;
  Int16 max = rows - size - start;

  if (max<0) max=0;

  if (last>rows-1) last=rows-1;
  for (row=start; row<TblGetNumberOfRows(tblP); row++)
    TblSetRowUsable(tblP, row, (row>=first && row<=last) ? true : false);

  SclSetScrollBar(sclP,first-start,0,max,size);
  TblMarkTableInvalid(tblP);
  TblRedrawTable(tblP);
}

/**
 * fixture form functions
 */
static void InitFormBl(void)
{
  Int16 i, fixup = 0, fixupX = 0, offset = 0, rows;
  ControlPtr cPtr;

  for (i=0; i<12; i++) {
    match[i].t1 = Id2Ptr(ID_FrmTeam+i);
    match[i].t2 = Id2Ptr(ID_FrmOpp+i);
    match[i].r1 = Id2Ptr(ID_FrmRes+i);
    match[i].r2 = Id2Ptr(ID_FrmOppRes+i);
    FldSetUsable(match[i].t1, false);
    FldSetUsable(match[i].t2, false);
    FldSetUsable(match[i].r1, false);
    FldSetUsable(match[i].r2, false);
    CtlSetUsable(Id2Ptr(ID_FrmBlPenaltyTrigger+i), false);
    cPtr = Id2Ptr(ID_FrmBlDateButton+i);
    CtlSetUsable(cPtr, false);
    switch (matchSymbol) {
    case 0:
      CtlSetLabel(cPtr,"-");
      break;
    case 1:
      CtlSetLabel(cPtr, "\012");
      cPtr->font = 3;
      break;
    case 2:
      CtlSetLabel(cPtr,"vs");
    }
  }

  rows = prefs.NumTeams>>1;
  if (rows == 11) fixup = 1;
  if (rows == 10) fixup = 2;
  if (rows  < 10) { 
    fixup = 2;
    offset = 5;
  }

  if (prefs.pointsPenalty == 0) { 
    fixupX = 1;
    for (i=0; i<12; i++) {
      FrmResizeObject(ID_FrmTeam+i,62,10+fixup);
      FrmResizeObject(ID_FrmOpp+i,59,10+fixup);
    }
  }

  if (fixup || fixupX) {
    for (i=0; i<12; i++) {
      FrmMoveObject(ID_FrmTeam+i,0,i*fixup+offset);
      FrmMoveObject(ID_FrmOpp+i,fixupX*5,i*fixup+offset);
      FrmMoveObject(ID_FrmRes+i,fixupX*11,i*fixup+offset);
      FrmMoveObject(ID_FrmOppRes+i,fixupX*13,i*fixup+offset);
      FrmMoveObject(ID_FrmBlDateButton+i,fixupX*5,i*fixup+offset);
      FrmMoveObject(ID_FrmBlPenaltyTrigger+i,0,i*fixup+offset);
      FrmMoveObject(ID_FrmBlPenaltyTrigger+20+i,0,i*fixup+offset);
    }
  }

  if (prefs.options & needWideResultField) {
    for (i=0; i<12; i++) {
      FrmMoveObject(ID_FrmRes+i,-10,0);
      FrmMoveObject(ID_FrmOppRes+i,-5,0);
      FrmResizeObject(ID_FrmRes+i,15,10+fixup);
      FrmResizeObject(ID_FrmOppRes+i,14,10+fixup);
    }
  }

}

/*
 * set "date/matchday switching button" with date of match[n]
 */
static void SetDateLabel(Int8 n)
{
  DateTimeType dateP;
  char label[17];

  showDay = false;

  if (match[n].Date > 0) {
    TimSecondsToDateTime(match[n].Date, &dateP);
    DateToAscii(dateP.month,dateP.day,dateP.year,
	        PrefGetPreference(prefDateFormat),
		label);

    // don't draw 0:00 time
    if (dateP.hour+dateP.minute>0) {
      StrCat(label," ");
      TimeToAscii(dateP.hour,dateP.minute,
		  PrefGetPreference(prefTimeFormat),
		  label+StrLen(label));
    }
  } else {
    StrCopy(label, "");
  }
  ButtonSetLabel(ID_FrmLabelMatchday,label);
}

/*
 * toggle between match day and most used date of match day
 */
void toggleMatchdayLabel(Boolean toggle, Int16 ctlId)
{
  char label[17];
  char md[3];

  if (toggle)
    showDay = !showDay;

  if (showDay || !DateOfMatchday) {
    char *p = GetString(ID_tSTR_Matchday);
    StrIToA(md, matchDay);
    StrCopy(label,p);
    StrCat(label, " ");
    StrCat(label, md);
  } else {
    DateTimeType dateP;
    TimSecondsToDateTime(DateOfMatchday, &dateP);
    DateToAscii(dateP.month,dateP.day,dateP.year,
		PrefGetPreference(prefDateFormat), label);
  }
  ButtonSetLabel(ctlId,label);
}

static void SetResultHandle(FieldType* fptr, UInt8 n1, UInt8 n2)
{
  MemHandle h;
  if (!(h = FldGetTextHandle(fptr))) {
    h = MemHandleNew(4);
  }
  if (n1==255 && n2==255)
    StrCopy(MemHandleLock(h),"");
  else
    StrIToA(MemHandleLock(h), n1);
  MemHandleUnlock(h);
  FldSetTextHandle(fptr, h);
}

/*
 * Get the most used date from one match day.
 * hmm; way too long for such a simple thing :-/
 */
UInt32 GetMostUsedDate(void)
{
  struct count {
    UInt32 date;
    Int8 num;
  } date[12];

  UInt32 Date;
  Int16 num = 0, i, j, numMatches;
  Boolean newDate;
  char *p, *q;
  matchType* m;

  p = q = MemHandleLock(DmQueryRecord(db,matchDay+1));
  numMatches = (Int8)*p;
  p += 2;

  for (i=0; i<numMatches; i++) {
    m = (matchType*)p;
    if (m->Date == 0)
      continue;
    newDate = true;
    for (j=0; j<num; j++) {
      if (date[j].date == m->Date) {
	date[j].num++;
	newDate = false;
	break;
      }
    }
    if (newDate) {
      date[num].date = m->Date;
      date[num].num = 1;
      num++;
    }
    p += sizeof(matchType);
  }

  MemPtrUnlock(q);

  Date = 0;
  i = 0;
  for (j=0; j<num; j++) {
    if (date[j].num > i) {
      Date = date[j].date;
      i = date[j].num;
    }
  }
  return Date;
}

static void RedrawForm(FormType* frm)
{
  FrmDrawForm(frm);
  DrawButton(ID_FrmBlSwitchBar+0,ID_Bitmap+3);
  DrawButton(ID_FrmBlSwitchBar+1,ID_Bitmap+4);
  DrawButton(ID_FrmBlSwitchBar+2,ID_Bitmap+6);
  DrawButton(ID_FrmBlSwitchBar+3,ID_Bitmap+8);
}

void DrawMatches(Boolean redraw)
{
  Int16 i;
  Int8 numTeams, numHandicaps;
  char *p, *q;
  matchType *m;
  FormPtr frm = FrmGetActiveForm();
  ListPtr pLst;

  p = q = MemHandleLock(DmQueryRecord(db,matchDay+1));

  numTeams = (Int8)*p++;
  numHandicaps = (Int8)*p++;

  for (i=0; i<12; i++) {

    match[i].team1 = match[i].team2 = match[i].Date = 0;

    if (i<numTeams) {
      m = (matchType*)p;
      match[i].Date = m->Date;
      match[i].penalty = m->penalty;
      if (match[i].penalty > 2) 
	match[i].penalty = 0;
    
      if ( m->t1==0 || m->t2==0 || 
	   m->t1>prefs.NumTeams || m->t2>prefs.NumTeams ) {
	DisplayErrorMessageAndStop(ID_ErrorMessages);
	MemPtrUnlock(q);
	prefs.Version = 0; // don't use this file again...
	return;
      }

      FldSetTextPtr(match[i].t1, theTeams[m->t1-1]);
      FldSetTextPtr(match[i].t2, theTeams[m->t2-1]);
      
      SetResultHandle(match[i].r1,m->r1,m->r2);
      SetResultHandle(match[i].r2,m->r2,m->r1);
      
      match[i].team1 = m->t1;
      match[i].team2 = m->t2;

      if (prefs.pointsPenalty) {
	pLst = Id2Ptr(ID_FrmBlPenaltyTrigger+20+i);
	LstSetSelection(pLst, m->penalty);
	ButtonSetLabel(ID_FrmBlPenaltyTrigger+i,
		       LstGetSelectionText(pLst,LstGetSelection(pLst)));
	CtlSetUsable(Id2Ptr(ID_FrmBlPenaltyTrigger+i), true);
      }

      p += sizeof(matchType);

      FldSetUsable(match[i].t1, true);
      FldSetUsable(match[i].t2, true);
      FldSetUsable(match[i].r1, true);
      FldSetUsable(match[i].r2, true);
      CtlSetUsable(Id2Ptr(ID_FrmBlDateButton+i), true);
    } else {
      
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmTeam+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmOpp+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmRes+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmOppRes+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmBlDateButton+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmBlPenaltyTrigger+i));
    }
  }

  DateOfMatchday = GetMostUsedDate();

  MemPtrUnlock(q);

  if (redraw) {
    RedrawForm(frm);
  }
  SelectedDB(DBNo, &info);
  CtlSetLabel(Id2Ptr(ID_FrmPopup), info.dbName);
  toggleMatchdayLabel(false,ID_FrmLabelMatchday);
}

/*
 * get the contents of the editable fields in the form 
 * and save them to database.
 */
static void SaveMatchdayResults(void)
{
  Int16 i;
  char *r1, *r2;
  char *p, *q;
  Int8 numTeams;
  Boolean dirty = false;
  matchType m;
  Int8 e1, e2;

  if (!db || prefs.options & readonly)
    return;

  FrmSetFocus(FrmGetActiveForm(), -1);

  p = q = MemHandleLock(DmGetRecord(db,matchDay+1));
  numTeams = (Int8)*p;
  p += 2;

  for (i=0; i<numTeams; i++) {
    if (i>=12)
      break;

    MemMove(&m,p,sizeof(matchType));

    r1 = FldGetTextPtr(match[i].r1);
    r2 = FldGetTextPtr(match[i].r2);

    e1 = e2 = -1;

    if (r1 && StrLen(r1) && r2 && StrLen(r2)) {
      e1 = StrAToI(r1);
      e2 = StrAToI(r2);
    }

    if (e1 != m.r1 || e2 != m.r2 || match[i].penalty != m.penalty) {
      m.r1 = e1;
      m.r2 = e2;
      m.penalty = match[i].penalty;
      DmWrite(q,2+i*sizeof(matchType),&m,sizeof(matchType));
      dirty = true;
    }
    p += sizeof(matchType);
  }
  MemPtrUnlock(q);
  DmReleaseRecord(db,matchDay+1,dirty);
}

Boolean MatchdayFormHandleEvent(EventPtr e)
{
  Boolean handled;
  Int16 t1, t2, item;
  FieldType *r1p, *r2p;
  ListPtr pLst;
  static FormType *frm;
  
  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {

  case frmOpenEvent:
    frm = FrmGetActiveForm();
    InitFormBl();
    ChooseDB();
    DrawMatches(true);
    ReturnToForm = ID_FrmBl;
    pLst = Id2Ptr(ID_FrmPopup+1);
    LstSetSelection(pLst,DBNo);
    LstMakeItemVisible(pLst,DBNo);
    handled = true; 
    break;

  case frmUpdateEvent:
    RedrawForm(frm);
    handled = true;
    break;

  case menuEvent:
    SaveMatchdayResults();
    switch(e->data.menu.itemID) {

    // About Bundesliga
    case ID_MenuItem:
      FrmPopupForm(ID_FrmAbout);
      break;

    // Edit team names, set fav team
    case ID_MenuItem + 1:
      FrmPopupForm(ID_FrmTeams);
      break;

    // Edit Fixtures
    case ID_MenuItem + 2:
      FrmPopupForm(ID_FrmEdit);
      break;

    // Delete Database
    case ID_MenuItem + 3:
      if (FrmAlert(ID_FrmDBDelete) == 0) { /* Delete */
        TableRememberPosFirst = 1;
	if ( DeleteDatabase(DBNo) 
	     == (appErrorClass|noDatabaseInstalled) ) {
	  DisplayErrorMessageAndStop(ID_ErrorMessages+5);
	} else {
	  DBNo = 0;
	  SelectedDB(DBNo, &info);
	  OpenDatabase(0, 0);
	  matchDay = prefs.matchDay;
	  DrawMatches(false);
	  FrmGotoForm(ID_FrmBl);
	}
      }
      break;

    // Create new database
    case ID_MenuItem + 4:
      FrmPopupForm(ID_FrmNewDB);
      break;

    // Edit league names, order of leagues
    case ID_MenuItem + 5:
      FrmPopupForm(ID_FrmLeague);
      break;

    // Settings form
    case ID_MenuItem + 6:
      FrmGotoForm(ID_FrmSettings);
      break;

    // Edit handicaps
    case ID_MenuItem + 7:
      stats.first = 0;
      stats.last = prefs.NumMatchDays-1;
      FrmPopupForm(ID_FrmHandicap);
      break;

    // Beam database
    case ID_MenuItem + 8:
      BeamDatabase(DBNo);
      break;

    // DB Settings form
    case ID_MenuItem + 9:
      FrmGotoForm(ID_FrmDatabaseSettings);
      break;

    case ID_MenuItem + 10:
      MyDoDialog(ID_FrmHelp, ID_FrmBl, "Tipps", FrmHelpHandleEvent, InitFrmHelp);
      break;
    }
    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlEnter.controlID) {

    // Matchday/Date label upper right corner
    case ID_FrmLabelMatchday:  
      toggleMatchdayLabel(true,ID_FrmLabelMatchday);
      break;

    // Table button
    case ID_FrmBlSwitchBar+1:
      FrmGotoForm(ID_FrmTable);
      break;

    // Table button
    case ID_FrmBlSwitchBar+2:
      FrmGotoForm(ID_FrmShowFixture);
      break;

    case ID_FrmButton: /* prev match day button */
      if (matchDay>1) {
	SaveMatchdayResults();
        --matchDay;
        DrawMatches(true);
      }
      break;
    
    case ID_FrmButton+1: /* next match day button */
      if (matchDay < prefs.NumMatchDays) {
        SaveMatchdayResults();
        ++matchDay;
        DrawMatches(true);
      }
      break;

    case ID_FrmBlSwitchBar+3: /* jump to button */
      FrmGotoForm(ID_FrmGoto);
      break;

    // select a different database/league file
    popup:
    case ID_FrmPopup:
      pLst = Id2Ptr(ID_FrmPopup+1);
      LstPopupList(pLst);
      item = LstGetSelection(pLst);
      if ( item != DBNo && item != noListSelection ) {
	TableRememberPosFirst = 1;
	DBNo = item;
	SaveMatchdayResults();
	SelectedDB(DBNo, &info);
	SaveStatus(DBNo);
	if ( OpenDatabase(info.cardNo, info.dbID) ) {
	  DBNo = 0;
	  SelectedDB(DBNo, &info);
	  OpenDatabase(0,0);
	}
	matchDay = prefs.matchDay;
	lastSelection = 0; // start with "all matches" in table view.
	DrawMatches(false);
	FrmGotoForm(ID_FrmBl);
      } else {
	SelectedDB(DBNo, &info);
	CtlSetLabel(Id2Ptr(ID_FrmPopup), info.dbName);
	CtlDrawControl(Id2Ptr(ID_FrmLabelMatchday));
      }
      break;

    default:
      // one of the "-" buttons between the team names
      if (e->data.ctlEnter.controlID >= ID_FrmBlDateButton &&
	  e->data.ctlEnter.controlID <= ID_FrmBlDateButton+11) {
	SetDateLabel(e->data.ctlEnter.controlID - ID_FrmBlDateButton);
      }

      if (e->data.ctlEnter.controlID >= ID_FrmBlPenaltyTrigger &&
	  e->data.ctlEnter.controlID <= ID_FrmBlPenaltyTrigger+11) {
	ListPtr pLst = Id2Ptr(e->data.ctlEnter.controlID+20);
	LstPopupList(pLst);
        ButtonSetLabel(e->data.ctlEnter.controlID,
		       LstGetSelectionText(pLst,LstGetSelection(pLst)));
	match[e->data.ctlEnter.controlID-ID_FrmBlPenaltyTrigger].penalty 
	  = LstGetSelection(pLst);
      }
    }
    handled = true;
    break;

  case keyDownEvent:
    if (e->data.keyDown.chr == vchrPageUp) { 
      SaveMatchdayResults();
      matchDay--;
      if (matchDay == 0)
	matchDay = prefs.NumMatchDays;
      DrawMatches(true);
      handled = true;
    }

    if (e->data.keyDown.chr == vchrPageDown) { 
      SaveMatchdayResults();
      matchDay++;
      if (matchDay > prefs.NumMatchDays)
	matchDay = 1;
      DrawMatches(true);
      handled = true;
    }

    if (e->data.keyDown.chr == vchrRockerLeft ||
	e->data.keyDown.keyCode == fiveWayLeft) {
      FrmGotoForm(ID_FrmShowFixture);
      handled = true;
    }
    
    if (e->data.keyDown.chr == vchrRockerRight || 
	e->data.keyDown.keyCode == fiveWayRight) {
      FrmGotoForm(ID_FrmTable);
      handled = true;
    }

    if (e->data.keyDown.chr == vchrRockerCenter || 
	(e->data.keyDown.keyCode == fiveWaySelect && 
	 (e->data.keyDown.modifiers & autoRepeatKeyMask) == 0) ) {
      goto popup;
    }
    break;

  // tapped on a team name
  case fldEnterEvent:
    item = e->data.fldEnter.fieldID-ID_FrmTeam;
    t1 = 0;
    r1p = r2p = 0;
    if (item>=0 && item<12) {
      t1 = match[item].team1;
      t2 = match[item].team2;
      r1p = match[item].r1;
      r2p = match[item].r2;
    } else if ( (item += ID_FrmTeam-ID_FrmOpp)>=0 
	       && item<12) {
      t1 = match[item].team2;
      t2 = match[item].team1;
      r1p = match[item].r2;
      r2p = match[item].r1;
    }
    if (t1>0) {
      handled = true;
      // if editMode, increase result of the team
      if ( (appPrefs.options & editMode) && !(prefs.options & readonly) ) {
	Int16 r1 = 0, r2 = 0, n1, n2;
	// suggested by Rolf Muth <rolf.muth@ruhr-uni-bochum.de>
	if ( r1p && FldGetTextLength(r1p)) 
	  r1 = ( StrAToI(FldGetTextPtr(r1p)) + 1 ) % 255;
	if ( r2p )
	  r2 = StrAToI(FldGetTextPtr(r2p)) % 255;
	n1 = r1 + (r2<<8);
	n2 = r2 + (r1<<8);
	SetResultHandle(r1p,n1,n2);
	SetResultHandle(r2p,n2,n1);
	//FrmDrawForm(FrmGetActiveForm());
	FldDrawField(r1p);
	FldDrawField(r2p);
      } else { // if not editMode, show statistics
	stats.team = t1;
	stats.first = 0;
        stats.last = prefs.NumMatchDays-1;
	stats.evalHandicap = true;
	stats.predicate = inBetween;
	if (ComputeTable(0xffffffff))
	  FrmGotoForm(ID_FrmStatistics);
      }
    }
    break;

  case frmCloseEvent:
    SaveMatchdayResults();
    break;

  default:
    break;
  }

  return handled;
}

/**
 * UI jump to match day functions
 */

static void InitFormGoto(void)
{
  Int16 row, col;
  gotoTable = Id2Ptr(ID_FrmGoto);
  for (row=0; row<TblGetNumberOfRows(gotoTable); row++) {
    TblSetRowUsable(gotoTable, row, 8*row>=prefs.NumMatchDays?false:true);
    TblSetRowSelectable(gotoTable, row, 
			8*row>=prefs.NumMatchDays?false:true);
    for (col=0; col<8; col++) {
      TblSetColumnUsable(gotoTable, col, true);
      if (row*8+col < prefs.NumMatchDays) {
        TblSetItemStyle(gotoTable, row, col, numericTableItem);
        TblSetItemInt(gotoTable, row, col, row*8+col+1);
      } else
        TblSetItemStyle(gotoTable, row, col, customTableItem);
    }
  }
}

Boolean GotoFormHandleEvent(EventPtr e)
{
  UInt32 now, selected; 
  DateTimeType date;
  Int16 r, c;
  char *p, *q;
  matchType *m;
  Boolean handled;
  Int8 numTeams;
  
  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {

  case frmOpenEvent:
    InitFormGoto();
    FrmDrawForm(FrmGetActiveForm());
    TblDrawTable(gotoTable);
    handled = true; 
    break;

  case tblSelectEvent:
    r = e->data.tblSelect.row;
    c = e->data.tblSelect.column;
    if (r*8+c+1 <= prefs.NumMatchDays) {
      matchDay = r*8 + c + 1;
      FrmGotoForm(ReturnToForm);
    }
    handled = true;
    break;

  case ctlSelectEvent:
    switch (e->data.ctlEnter.controlID) {

    case ID_FrmGoto+1:

      TimSecondsToDateTime(TimGetSeconds(), &date);
      p = GetString(ID_tSTR_ChooseDate);

      if ( SelectDay(selectDayByDay,
                     &date.month, &date.day, &date.year, p) ) {
	now = TimDateTimeToSeconds(&date);

	selected = 0xffffffff;
	matchDay = 1;

	for (r=0; r<prefs.NumMatchDays; r++) {
	  
	  p = q = MemHandleLock(DmQueryRecord(db,r+2));
	  numTeams = (Int8)*p;
	  p += 2;
	  
	  for (c=0; c<numTeams; c++) {
	    m = (matchType*)p;
	    if ( m->Date >= now && m->Date < selected) {
	      selected = m->Date;
	      matchDay = r+1;
	    }
	    p += sizeof(matchType);
	  }
	  MemPtrUnlock(q);
	}
	FrmGotoForm(ReturnToForm);
      }

      handled = true;
      break;

    case ID_FrmGoto+2:
      FrmGotoForm(ReturnToForm);
      handled = true;
      break;
    }
    break;

  default:
    break;
  }

  return handled;
}

/**
 * Credits Form (via (i) in About Form)
 */
Boolean AboutFormHandleEvent(EventPtr e)
{
  Boolean handled;
  FormPtr frm;
  FieldPtr fld;

  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {

  case frmOpenEvent:
    if (!progDatePtr) {
      Int16 len;
      Char* template;
      switch (PrefGetPreference(prefLongDateFormat)) {
      case dfMDYLongWithComma:
	template = "^2l ^0r, ^4r";
	break;
      case dfDMYLong:
	template = "^2l ^0r ^4r";
	break;
      case dfDMYLongWithDot:
	template = "^0r. ^2l ^4r";
	break;
      case dfDMYLongNoDay:
	template = "^2l ^4r";
	break;
      case dfDMYLongWithComma:
	template = "^0r ^2l, ^4r";
	break;
      case dfYMDLongWithDot:
	template = "^4r.^3r.^0r";
	break;
      default:
	template = "^2l ^0r ^4r";
      }
      len = DateGlueTemplateToAscii(template,
                                    tIntProgDateMonth,
                                    tIntProgDateDay,
                                    tIntProgDateYear,
                                    0,0);
      progDatePtr = (Char*)MemPtrNew(len+1);
      DateGlueTemplateToAscii(template,
                              tIntProgDateMonth,
                              tIntProgDateDay,
                              tIntProgDateYear,
                              progDatePtr,len+1);
    }
    frm = FrmGetActiveForm();
    fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, ID_FrmAboutButton+1));
    FldSetTextPtr(fld, progDatePtr);
    FrmDrawForm(FrmGetActiveForm());
    handled = true;
    break;

  case ctlSelectEvent:
    if (e->data.ctlSelect.controlID == ID_FrmAboutButton) {
      FrmReturnToForm(0);
      handled = true;
    }
    break;

  case penDownEvent:
    // this is the position of the (i)
    if (e->screenX >= 144 && e->screenX <= 155 &&
	e->screenY >= 0 && e->screenY <= 11) {
      // this way we don't have to use the default bold font.
      MyDoDialog(ID_FrmHelp, ID_tSTR_Credits, "Credits", FrmHelpHandleEvent, InitFrmHelp);
      handled =true;
    }

  default:
    break;
  }

  return handled;
}

/**
  * Statistics Form Handling Routines
  */
static void DrawIntAt(Int16 v, Coord x, Coord y)
{
  Char buf[35];
  StrIToA(buf,v);
  WinDrawChars(buf,StrLen(buf),x,y);
}

/*
 * build up a string holding the results of matches
 * of a team as characters in terms of won/draw/lost
 * home/away (huh?)
 */
static void GetGamesString(Char* buf, Int16 n)
{
  Int16 i, j;
  Int8 numTeams;
  char *p, *q, *r;
  matchType *m;


  StrCopy(buf,"");
  for (i=stats.first; i<=stats.last; i++) {

    p = q = MemHandleLock(DmQueryRecord(db,i+2));
    numTeams = (Int8)*p;
    p+=2;

    for (j=0; j<numTeams; j++) {
      m = (matchType*)p;
      if (m->t1 == n || m->t2 == n) {

	if (stats.predicate(m,n,i)) {
	  if (m->r1 != 0xff && m->r2 != 0xff) {
	    if (m->t1 == n) { // home match
	      if (m->r1>m->r2) {
		r = GetString(ID_tSTR_H_WON);
	      } else if (m->r1 == m->r2) {
		r = GetString(ID_tSTR_H_DRAW);
	      } else {
		r = GetString(ID_tSTR_H_LOST);
	      }
	    } else { // away match
	      if (m->r2 > m->r1) {
		r = GetString(ID_tSTR_A_WON);
	      } else if (m->r1 == m->r2) {
		r = GetString(ID_tSTR_A_DRAW);
	      } else {
		r = GetString(ID_tSTR_A_LOST);
	      }
	    }
	    StrCat(buf,r);
	  } else {
	    StrCat(buf, "-");
	  }
	}
      }
      p += sizeof(matchType);
    }
    MemPtrUnlock(q);
  }
}

Boolean StatisticsFormHandleEvent(EventPtr e)
{
  Boolean handled;
  Char *title;
  Char buf[91];
  Char Position[3];
  Int16 i, k, team=0;
  
  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {

  case frmOpenEvent:
    TeamResult = MemHandleLock(TeamResultHandle);

    for (i=0; i<prefs.NumTeams; i++) {
      if (stats.team == TeamResult[i].teamNo) {
	team = i;
	break;
      }
    }

    FntSetFont(0);
    // modify title, set team name
    title = (char*)FrmGetTitle(FrmGetActiveForm());
    while ( *title++ != ' ' );
    MemMove(title, theTeams[stats.team-1], 
	    StrLen(theTeams[stats.team-1]));
    StrIToA(Position, TeamResult[team].Position+1);
    FldSetTextPtr(Id2Ptr(ID_FrmStatisticsFld), Position);
    FrmDrawForm(FrmGetActiveForm());


    // the rest is done with WinDrawChars because it
    // has rather positive effects on code size; drawback:
    // it's really ugly :-)
    DrawIntAt(TeamResult[team].Matches,5,37);
    DrawIntAt(TeamResult[team].Won,20,37);
    DrawIntAt(TeamResult[team].Rem,35,37);
    DrawIntAt(TeamResult[team].Lost,50,37);
    StrIToA(buf,TeamResult[team].Goals);
    StrCat(buf,":");
    StrIToA(buf+StrLen(buf),TeamResult[team].AGoals);
    WinDrawChars(buf,StrLen(buf),65,37);
    DrawIntAt(TeamResult[team].Goals-TeamResult[team].AGoals,95,37);
    DrawIntAt(TeamResult[team].Points,115,37);
    
    DrawIntAt(TeamResult[team].HMatches,5,71);
    DrawIntAt(TeamResult[team].HWon,20,71);
    DrawIntAt(TeamResult[team].HRem,35,71);
    DrawIntAt(TeamResult[team].HLost,50,71);
    StrIToA(buf,TeamResult[team].HGoals);
    StrCat(buf,":");
    StrIToA(buf+StrLen(buf),TeamResult[team].HAGoals);
    WinDrawChars(buf,StrLen(buf),65,71);
    DrawIntAt(TeamResult[team].HGoals-TeamResult[team].HAGoals,95,71);
    DrawIntAt(TeamResult[team].HPoints,115,71);

    DrawIntAt(TeamResult[team].Matches-TeamResult[team].HMatches,5,105);
    DrawIntAt(TeamResult[team].Won-TeamResult[team].HWon,20,105);
    DrawIntAt(TeamResult[team].Rem-TeamResult[team].HRem,35,105);
    DrawIntAt(TeamResult[team].Lost-TeamResult[team].HLost,50,105);
    StrIToA(buf,TeamResult[team].Goals-TeamResult[team].HGoals);
    StrCat(buf,":");
    StrIToA(buf+StrLen(buf),
	    TeamResult[team].AGoals-TeamResult[team].HAGoals);
    WinDrawChars(buf,StrLen(buf),65,105);
    DrawIntAt(TeamResult[team].Goals-TeamResult[team].AGoals
	      -TeamResult[team].HGoals+TeamResult[team].HAGoals,95,105);
    DrawIntAt(TeamResult[team].Points-TeamResult[team].HPoints,115,105);

    GetGamesString(buf,stats.team);
    k = prefs.NumMatchDays>>1;
    WinDrawChars(buf,(i=StrLen(buf))>k?k:i,5,117);
    if (i>k)
      WinDrawChars(buf+k,StrLen(buf+k),5,128);
    
    handled = true; 
    MemHandleUnlock(TeamResultHandle);
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {

    // return to either table or results form.
    case ID_FrmStatisticsButton:
      FrmGotoForm(ReturnToForm);
      handled = true; 
      break;

    // show team fixture.
    case ID_FrmStatisticsButton+1:
      FrmGotoForm(ID_FrmShowFixture);
      handled = true; 
      break;
    }
    break;

  case penDownEvent:
    // this is the position of the (i)
    if (e->screenX >= 144 && e->screenX <= 155 &&
	e->screenY >= 0 && e->screenY <= 11) {
      // this way we don't have to use the default bold font.
      MyDoDialog(ID_FrmHelp, ID_FrmStatistics, "Tipps", FrmHelpHandleEvent, InitFrmHelp);
      handled =true;
    }

  default:
    break;
  }

  return handled;
}

