/*
 * Bundesliga is Copyright (c) 1997-2003 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <PalmOS.h>

#include "bl.h"
#include "blrcp.h"

struct {
  FieldType *t1, *t2;
  UInt32 date;
} editMatch[12];

static RectangleType eRect;
static PointType ePt;
static DateTimeType date;
static Int16 editMatchDay;
static FieldType *lastField;

static void SetDateMatchdayLabel(void) EDITFUNC;
static void SetDateMatchday(Int8) EDITFUNC;
static void InitFormEdit(void) EDITFUNC;
static Boolean DrawFormEdit(void) EDITFUNC;
static void exchange(FieldType* team1, FieldType* team2) EDITFUNC;
static Int16 getY(Int16 y) EDITFUNC;
static FieldType* getTeamUnder(FieldType* f) EDITFUNC;
static FieldType* teamUnder(Coord x, Coord y) EDITFUNC;
static void dragTeam(Coord x, Coord y) EDITFUNC;
static Int16 StringToTeamNum(char* s) EDITFUNC;
static void UpdateFixture(Boolean,Int16,Boolean) EDITFUNC;
static void InitFormTeams(void) EDITFUNC;
static Int16 TeamsListGetSelection(void) EDITFUNC;
static void RedrawList(UInt16 ID_Form, UInt16 ID_List) EDITFUNC;
static void DrawFormEnterTeam(void) EDITFUNC;
static void SetEnteredTeam(Int16) EDITFUNC;
static void InitFrmNewDB(void) EDITFUNC;
static void ReadFormNewDBValues(void) EDITFUNC;
static void InitFrmRenameLeague(void) EDITFUNC;
static void RenameDatabaseFile(void) EDITFUNC;
static void updateDbNo(Int16 i, Int16 j) EDITFUNC;

/**
 * Edit Form
 * purpose: edit the fixture by drag and drop
 */
static void SetDateMatchdayLabel()
{
  static Char buf[3];
  ControlPtr ctlP = Id2Ptr(ID_FrmEditDate);
  char *label = (char*)CtlGetLabel(ctlP);
  CtlEraseControl(ctlP);
  DateToAscii(date.month, date.day, date.year, 
  	      PrefGetPreference(prefDateFormat), label);

  ctlP = Id2Ptr(ID_FrmEditDate+1);
  label = (char*)CtlGetLabel(ctlP);
  CtlEraseControl(ctlP);
  TimeToAscii(date.hour, date.minute,
  	      PrefGetPreference(prefTimeFormat), label);

  StrIToA(buf, editMatchDay);
  FldSetTextPtr(Id2Ptr(ID_FrmEditFld), buf);
}

static void SetDateMatchday(Int8 fld)
{
  UInt16 i;
  if (fld != -1) {
    editMatch[fld].date = TimDateTimeToSeconds(&date);
  } else {
    for (i=0; i<12; i++) {
      editMatch[i].date = TimDateTimeToSeconds(&date);
    }
  }
}

static void InitFormEdit(void)
{
  Int16 i, fixup=0, rows, offset=0;

  for (i=0; i<12; i++) {
    editMatch[i].t1 = Id2Ptr(ID_FrmEditTeam+i);
    editMatch[i].t2 = Id2Ptr(ID_FrmEditTeam+i+20);
  }

  rows = (prefs.NumTeams+1)>>1;
  if (rows == 11) fixup = 1;
  if (rows == 10) fixup = 2;
  if (rows  < 10) { 
    fixup = 2;
    offset = 5;
  }

  if (fixup) {
    for (i=0; i<12; i++) {
      FrmMoveObject(ID_FrmEditTeam+i,0,i*fixup+offset);
      FrmMoveObject(ID_FrmEditTeam+20+i,0,i*fixup+offset);
      FrmMoveObject(ID_FrmEditTeam+40+i,0,i*fixup+offset);
    }
    for (i=0; i<5; i++) {
      FrmMoveObject(ID_FrmEditButton+i,0,-fixup);
    }
  }
}

static Boolean DrawFormEdit(void)
{
  UInt32 mask = 0UL, Date = 0;
  Int16 i;
  Int8 numMatches;
  char *p, *q;
  matchType *m;
  Boolean hasResults = false;

  p = q = MemHandleLock(DmQueryRecord(db,editMatchDay+1));
  numMatches = (Int8)*p;
  p+=2;

  for (i=0; i<12; i++) {
    if (i<numMatches) {
      m = (matchType*)p;
      if (!Date)
	Date = m->Date;
    
      FldSetTextPtr(editMatch[i].t1, theTeams[m->t1-1]);
      FldSetUsable(editMatch[i].t1, true);
      FldSetTextPtr(editMatch[i].t2, theTeams[m->t2-1]);
      FldSetUsable(editMatch[i].t2, true);
      editMatch[i].date = m->Date;
      CtlSetUsable(Id2Ptr(ID_FrmEditTeam+40+i), true);

      /* remember which teams play this round */
      mask |= ( 1UL<<(m->t1-1) ) | ( 1UL<<(m->t2-1) );
      
      hasResults = hasResults || (m->r1 != 0xff && m->r2 != 0xff);
      
      p += sizeof(matchType);
    } else {
      FormType *frm = FrmGetActiveForm();
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmEditTeam+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmEditTeam+20+i));
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmEditTeam+40+i));
    }

  }

  lastField = editMatch[numMatches-1].t2;

  if (prefs.NumTeams % 2 == 1) {
    for (i=0; i<prefs.NumTeams; i++) {
      /* set team that doesn't play this round */
      if ( (mask & (1UL<<i)) == 0 ) { 
	FldSetTextPtr(editMatch[prefs.NumTeams>>1].t2,theTeams[i]);
	FldSetUsable(editMatch[prefs.NumTeams>>1].t2, true);
	lastField = editMatch[prefs.NumTeams>>1].t2;
	break;
      }
    }
  }
  MemPtrUnlock(q);

  if (Date == 0) {
    TimSecondsToDateTime(TimGetSeconds(), &date);
  } else {
    TimSecondsToDateTime(Date, &date);
  }
  SetDateMatchdayLabel();

  return hasResults;
}

/*
 * swap contents of the two fields
 */
static void exchange(FieldType* team1, FieldType* team2)
{
  char *swap1, *swap2;
  swap1 = FldGetTextPtr(team1);
  swap2 = FldGetTextPtr(team2);
  FldSetTextPtr(team1, swap2);
  FldSetTextPtr(team2, swap1);
  FldDrawField(team1);
  FldDrawField(team2);
}

static Int16 getY(Int16 y)
{
  Int16 div = 10 , offset = 0, rows = prefs.NumTeams>>1;

  if (rows==11)
    div = 11;
  else if (rows==10)
    div = 12;
  else if (rows<10) {
    div = 12;
    offset = 5;
  }

  y = (y-24-offset) / div;
  return y;
}

static FieldType* getTeamUnder(FieldType* f)
{
  FieldAttrType attr;
  FldGetAttributes(f, &attr);
  if (!attr.usable) /* is field used */
    return NULL;
  FldGetBounds(f, &eRect);
  return f;
}

static FieldType* teamUnder(Coord x, Coord y)
{
  if (y<24 || y>146 || (x>70 && x<88) || x>153 || x<5)
    return NULL;
  y = getY(y);
  if (y<0 || y>11)
    return NULL;

  if (x<70) { /* left half  */
    return getTeamUnder(editMatch[y].t1);
  }
  /* right half */
  return getTeamUnder(editMatch[y].t2);
}

static void dragTeam(Coord x, Coord y)
{
  WinInvertRectangle(&eRect, 2);
  RctOffsetRectangle(&eRect, x-ePt.x, y-ePt.y);
  WinInvertRectangle(&eRect, 2);
  ePt.x = x;
  ePt.y = y;
}

/*
 * return number of team in team table of argument "s"
 */
static Int16 StringToTeamNum(char* s)
{
  Int16 i;

  /*
   * We've only mangled pointers, so let's compare the pointers
   */
  for (i=0; i<prefs.NumTeams; i++) {
    if (theTeams[i] == s) {
      return i+1;
    }
  }
  return 0;
}

/*
 * correct settings in fixture according to drag & drop actions before;
 * if necessary reset results - the user was warned about this!
 * With all it's functionality now this routine kind of is a hack.
 */
static void UpdateFixture(Boolean addMatch, Int16 delMatchNum, Boolean KeepResults)
{
  matchType *m,mn;
  Int8 numTeams, numHandicaps, t1, t2;
  char *p, *q;
  Int16 i, size;

  if (prefs.options & readonly)
    return;

  p = MemHandleLock(DmQueryRecord(db,editMatchDay+1));
  numTeams = (Int8)*p;
  numHandicaps = (Int8)*(p+1);
  MemPtrUnlock(p);

  mn.t1 = 1;
  mn.t2 = 2;
  mn.r1 = -1;
  mn.r2 = -1;
  mn.Date = 0;

  if (numTeams>=9)
    addMatch = false;

  size = 2 + numTeams*sizeof(matchType) + numHandicaps*sizeof(handicapType);

  if (addMatch)
    size += sizeof(matchType);

  if (delMatchNum > -1)
    size -= sizeof(matchType);

  p = q = MemHandleLock(DmResizeRecord(db, editMatchDay+1, size));
  p += 2;

  for (i=0; i<numTeams; i++) {
    m = (matchType*)p;

    if (i == delMatchNum)
      continue;

    t1 = StringToTeamNum(FldGetTextPtr(editMatch[i].t1));
    t2 = StringToTeamNum(FldGetTextPtr(editMatch[i].t2));
    if (t1 && t2) {
      mn.t1 = t1; 
      mn.t2 = t2; 
      mn.Date = editMatch[i].date;
      if (KeepResults) {
	mn.r1 = m->r1;
	mn.r2 = m->r2;
      } else {
	mn.r1 = mn.r2 = 0xff;
      }
      DmWrite(q,p-q,&mn,sizeof(matchType));
    }
    p += sizeof(matchType);
  }

  if (addMatch) {
    numTeams++;
    DmWrite(q,0,&numTeams,1);
    DmWrite(q,p-q+sizeof(matchType),p,numHandicaps*sizeof(handicapType));
    DmWrite(q,p-q,&mn,sizeof(matchType));
  }

  if (delMatchNum > -1) {
    numTeams--;
    DmWrite(q,0,&numTeams,1);
    DmWrite(q,p-q,p+sizeof(matchType),numHandicaps*sizeof(handicapType));
  }

  MemPtrUnlock(q);
  DmReleaseRecord(db,editMatchDay+1,true);
}

Boolean EditFormHandleEvent(EventPtr e)
{
  Boolean handled;
  static FieldType* lastTeam;
  FieldType* newTeam;
  static ListType *pLst;
  static Boolean hasChanged = false;
  static Boolean canKeepResults = true;
  static Int16 dateButton;
  Int16 i, index;
  static FormType* frm;
  
  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {

  case frmOpenEvent:
    frm = FrmGetActiveForm();
    InitFormEdit();
    handled = true;
    editMatchDay = matchDay;
    dateButton = -1;
    if (DrawFormEdit()) {
      if (FrmAlert(ID_FrmHasResults) == 1) {
        /* don't delete results, but return to form immediately */
        FrmReturnToForm(0);
        break;
      }
    }
    FrmDrawForm(frm);
    pLst = Id2Ptr(ID_FrmEditList);
    LstSetHeight(pLst, prefs.NumTeams>10?10:prefs.NumTeams);
    LstSetListChoices(pLst, theTeams, prefs.NumTeams);
    break;

  case menuEvent:
    switch(e->data.menu.itemID) {
    case ID_EditMenuItem:    /* Add Match */
      UpdateFixture(true,-1,canKeepResults); /* write record and add a match */
      matchDay = editMatchDay;
      FrmGotoForm(ID_FrmEdit);
      handled = true;
      break;

    case ID_EditMenuItem+1: /* Delete Match */
      handled = true;
      if (dateButton == -1) {
	DisplayInformationMessage(ID_tSTR_ChooseMatch);
	break;
      }
      UpdateFixture(false,dateButton,canKeepResults);
      matchDay = editMatchDay;
      FrmGotoForm(ID_FrmEdit);
      break;

    case ID_EditMenuItem+2: /* Add Match Day */
      if (prefs.NumMatchDays<90) {
	index = editMatchDay+3;
	prefs.NumMatchDays++;
	InsertNewRecord(db, &index, prefs.NumTeams>>1);
      }
      handled = true;
      break;

    case ID_EditMenuItem+3: /* Delete shown Match Day */
      if (prefs.NumMatchDays>1) {
	prefs.NumMatchDays--;
	DeleteRecord(db,editMatchDay+1);
	editMatchDay = matchDay = editMatchDay - 1;
	if (matchDay == 0) {
	  editMatchDay = matchDay = 1;
	}
      }
      FrmGotoForm(ID_FrmEdit);
      handled = true;
      break;

    case ID_EditMenuItem+4: /* Add Team */
      // diese Geschichte ist Buggy unter der DIA-API, weil
      // kein Eventhandler installiert ist, der ein Resize-Event
      // verarbeitet.
      if (prefs.NumTeams < 24) {
	FormPtr dialogFrm = FrmInitForm(ID_FrmEnterTeam);
	FrmSetActiveForm(dialogFrm);
	SetResizePolicy(ID_FrmEnterTeam);
	ResizeRefreshCurrentForm();
	FrmDrawForm(dialogFrm);
	if (FrmDoDialog(dialogFrm) == ID_FrmEnterTeamButton) {
	  char *team = FldGetTextPtr(Id2Ptr(ID_FrmEnterTeam));
	  if (team && StrLen(team)) {
	    prefs.NumTeams++;
	    if (CtlGetValue(Id2Ptr(ID_FrmEnterTeamButton+2))) {
	      prefs.favTeam = prefs.NumTeams;
	    }
	    SavePreferences();
	    MemHandleResize(TeamResultHandle, prefs.NumTeams*sizeof(TeamResultType));
	    SetEnteredTeam(prefs.NumTeams-1);
	    matchDay = editMatchDay;
	    FrmGotoForm(ID_FrmEdit);
	  }
	}
	FrmDeleteForm(dialogFrm);
	FrmSetActiveForm(frm);
      }
      handled = true;
      break;
    }
    break;

  case penDownEvent:
    /* are we over a field with a team? */
    if ((lastTeam = teamUnder(e->screenX, e->screenY))) {
      WinInvertRectangle(&eRect, 2);
      ePt.x = e->screenX;
      ePt.y = e->screenY;
      handled = true;
    }
    // this is the position of the (i)
    if (e->screenX >= 144 && e->screenX <= 155 &&
	e->screenY >= 0 && e->screenY <= 11) {
      // this way we don't have to use the default bold font.
      MyDoDialog(ID_FrmHelp, ID_FrmEditHelp, "Tipps", FrmHelpHandleEvent, InitFrmHelp);
      handled =true;
    }

    break;

  case penMoveEvent:
    /* move rectangle */
    if (lastTeam) {
      dragTeam(e->screenX, e->screenY);
      handled = true;
    }
    break;

  case penUpEvent:
    /* swap teams */
    if (lastTeam) {
      WinInvertRectangle(&eRect, 2);
      if ((newTeam = teamUnder(e->screenX, e->screenY))) {
	exchange(lastTeam,newTeam);
        hasChanged = true;
	canKeepResults = false;
	handled = true;
      }
    }
    break;

  case ctlSelectEvent:
    /* "Ready" */
    if (e->data.ctlSelect.controlID == ID_FrmEditButton) {
      if (hasChanged) {
        UpdateFixture(false,-1,canKeepResults);
      }
      FrmReturnToForm(0);
      if (hasChanged) 
	DrawMatches(true);
      handled = true; 
      break;
    }

    /* "Cancel" */
    if (e->data.ctlSelect.controlID == ID_FrmEditButton+1) {
      FrmReturnToForm(0);
      handled = true; 
      break;
    }

    /* "Left" / "Right" */
    if (e->data.ctlSelect.controlID == ID_FrmEditButton+2 ||
        e->data.ctlSelect.controlID == ID_FrmEditButton+3) {
      handled = true;
      if ( (editMatchDay > 1 
	    && e->data.ctlSelect.controlID == ID_FrmEditButton+2) 
         || (editMatchDay < prefs.NumMatchDays 
	     && e->data.ctlSelect.controlID == ID_FrmEditButton+3) ){
        if (hasChanged) {
          UpdateFixture(false,-1,canKeepResults);
          hasChanged = false;
	  canKeepResults = true;
        }
        editMatchDay += 
	  (e->data.ctlSelect.controlID == ID_FrmEditButton+2)?-1:1;
        if (DrawFormEdit()) {
          if (FrmAlert(ID_FrmHasResults) == 1) {
            editMatchDay += 
	      (e->data.ctlSelect.controlID == ID_FrmEditButton+2)?1:-1;
	    DrawFormEdit();
            break;
          }
        }
	if (dateButton != -1) {
	  ButtonSetLabel(ID_FrmEditTeamDateButton+dateButton, "-");
	  dateButton = -1;
	}
        FrmDrawForm(frm);
      }
      break;
    }

    /* "Date" */
    if (e->data.ctlSelect.controlID == ID_FrmEditDate) {
      char *r = GetString(ID_tSTR_ChooseDate);
      if ( SelectDay(selectDayByDay, 
		     &date.month, &date.day, &date.year, r) ) {
	SetDateMatchdayLabel();
	SetDateMatchday(dateButton);
        FrmDrawForm(frm);
        hasChanged = true;
      }
      handled = true; 
      break;
    }

    /* "Time" */
    if (e->data.ctlSelect.controlID == ID_FrmEditDate+1) {
      char *r = GetString(ID_tSTR_ChooseTime);
      if (FeatureSet31) {
	if ( SelectOneTime(&date.hour, &date.minute, r) ) {
	  SetDateMatchdayLabel();
	  SetDateMatchday(dateButton);
	}
      } else {
	/* grr; why isn't SelectOneTime() exported before 3.1? 
	 * The result looks really really ugly... */
	TimeType st, et;
	st.hours = date.hour;
	st.minutes = date.minute;
	if ( SelectTimeV33(&st, &et, false, r, 11) ) {
	  date.hour = st.hours;
	  date.minute = st.minutes;
	  SetDateMatchdayLabel();
	  SetDateMatchday(dateButton);
	}
      }
      FrmDrawForm(frm);
      hasChanged = true;
      handled = true; 
      break;
    }

    /* "Team" */
    if (e->data.ctlSelect.controlID == ID_FrmEditButton+4) {
      index=StringToTeamNum(FldGetTextPtr(editMatch[((prefs.NumTeams+1)>>1)-1].t2));
      LstSetSelection(pLst, index-1);
      LstPopupList(pLst);
      if ( noListSelection != (i = LstGetSelection(pLst)) ) {
        FldSetTextPtr(lastField, theTeams[i]);
        FldDrawField(lastField);
        hasChanged = true;
	canKeepResults = false;
      }
      handled = true;
      break;
    }

    /* One of the "Date" buttons. */
    if (e->data.ctlSelect.controlID >= ID_FrmEditTeamDateButton &&
	e->data.ctlSelect.controlID <= ID_FrmEditTeamDateButton+11) {
      if (dateButton != -1) {
	ButtonSetLabel(ID_FrmEditTeamDateButton+dateButton, "-");
      }
      dateButton = e->data.ctlSelect.controlID - ID_FrmEditTeamDateButton;
      ButtonSetLabel(ID_FrmEditTeamDateButton+dateButton, "=");
      TimSecondsToDateTime(editMatch[dateButton].date, &date);
      SetDateMatchdayLabel();
      FrmDrawForm(frm);
      handled = true;
      break;
    }

    break;

  default:
    break;
  }

  return handled;
}

/**
 * routines handling editing and entering teams list
 */
static void InitFormTeams()
{
  SelectedDBInfo info;
  ListPtr p;

  SelectedDB(DBNo, &info);
  FrmCopyLabel(FrmGetActiveForm(), ID_FrmTeamsName, info.dbName);

  p = FrmGetListPtr(ID_FrmTeams, ID_FrmTeamsList);
  LstSetListChoices(p, theTeams, prefs.NumTeams);
}

static Int16 TeamsListGetSelection()
{
  ListPtr p;
  p = FrmGetListPtr(ID_FrmTeams, ID_FrmTeamsList);
  return LstGetSelection(p);
}

static void RedrawList(UInt16 ID_Form, UInt16 ID_List)
{
  ListPtr p;
  p = FrmGetListPtr(ID_Form, ID_List);
  LstEraseList(p);
  LstDrawList(p);
}

Boolean TeamsFormHandleEvent(EventPtr e)
{
  Boolean handled;
  static ListPtr p;
  
  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {

  case frmOpenEvent:
    InitFormTeams();
    FrmDrawForm(FrmGetActiveForm());
    p = FrmGetListPtr(ID_FrmTeams, ID_FrmTeamsList);
    handled = true; 
    break;

  case ctlSelectEvent:
    /* Button "Ready" */
    if (e->data.ctlSelect.controlID == ID_FrmTeamsButton) {
      FrmReturnToForm(0);
      DrawMatches(true);
      handled = true; 
      break;
    }

    /* Button "Edit" */
    if (e->data.ctlSelect.controlID == ID_FrmTeamsButton+1) {
      FrmPopupForm(ID_FrmEnterTeam);
      handled = true; 
      break;
    }
    break;

  case keyDownEvent:
    switch (e->data.keyDown.chr) {
    case pageUpChr:
      LstScrollList(p,winUp,10);
      handled = true; 
      break;

    case pageDownChr:
      LstScrollList(p,winDown,10);
      handled = true; 
      break;
    }
    break;


  default:
    break;
  }

  return handled;
}

/**
 * routines handling the "Enter Team" form
 */
static void DrawFormEnterTeam()
{
  Int16 nr = TeamsListGetSelection();

  if (nr == noListSelection)
    return;

  SetField(ID_FrmEnterTeamFld,18,theTeams[nr]);

  CtlSetValue(Id2Ptr(ID_FrmEnterTeamButton+2), prefs.favTeam == nr+1);
}

static void SetEnteredTeam(Int16 nr)
{
  Int16 size = 0, i, index = 1;
  char *q, *s;

  if (prefs.options & readonly)
    return;

  theTeams[nr] = FldGetTextPtr(Id2Ptr(ID_FrmEnterTeam));
  for (i=0; i<prefs.NumTeams; i++) {
    size += StrLen(theTeams[i])+1;
  }

  /*
   * resizing the record would overwrite TeamResult, as we
   * actually use the pointers from the record. So we create
   * a new number "1" record and shift down all other records
   */
  q = MemHandleLock(DmNewRecord(db,&index,size));
  size = 0;
  for (i=0; i<prefs.NumTeams; i++) {
    s = theTeams[i];
    DmWrite(q,size,s,StrLen(s)+1);
    size += StrLen(s) + 1;
  }
  MemPtrUnlock(q);
  DmReleaseRecord(db,1,true);
  DmRemoveRecord(db,2);  // delete old number "1" record.
  ReadDatabase(db);
}

Boolean EnterTeamFormHandleEvent(EventPtr e)
{
  Boolean handled;
  static Boolean update = false;
  Int16 nr;
  
  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {

  case frmOpenEvent:
    DrawFormEnterTeam();
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    /* Ready */
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmEnterTeamButton:
      if ( noListSelection != (nr = TeamsListGetSelection()) ) {
        SetEnteredTeam(nr);
        update = true;
        if (CtlGetValue(Id2Ptr(ID_FrmEnterTeamButton+2))) {
	  prefs.favTeam = nr+1;
        } else if (prefs.favTeam == nr+1) {
          prefs.favTeam = 0;
        }
      }

    /* Cancel */
    case ID_FrmEnterTeamButton+1:
      FrmReturnToForm(0);
      if (update) { 
	InitFormTeams();
	update = false;
      }
      RedrawList(ID_FrmTeams, ID_FrmTeamsList);
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

  return handled;
}

/**
 * routines handling creation of a new database
 */
static void InitFrmNewDB()
{
  Int16 i, s[3] = { 19, 3, 3 };
  for (i=0; i<2; i++) {
    SetField(ID_FrmNewDBFld, s[i], "");
  }
}

/*
 * create database according to entries in form
 */
static void ReadFormNewDBValues()
{
  char *name, *nr, *rds;
  DmOpenRef db;
  BlDatabasePreferencesType newPrefs;
  Int16 n = 0, m = 0, i;

  nr = FldGetTextPtr(Id2Ptr(ID_FrmNewDBFld+1));
  rds = FldGetTextPtr(Id2Ptr(ID_FrmNewDBFld+2));

  if (nr)  n = StrAToI(nr);
  if (rds) m = StrAToI(rds);

  if (n < 2 || n > 24 || m < 1 || m > 90) {
    DisplayErrorMessage(ID_ErrorMessages+4);
    return;
  }

  name = FldGetTextPtr(Id2Ptr(ID_FrmNewDBFld));

  newPrefs.NumTeams      = n;
  newPrefs.NumMatchDays  = m;
  newPrefs.matchDay      = 1;
  newPrefs.Version       = 9;
  newPrefs.options       = 0;
  newPrefs.favTeam       = 0;
  newPrefs.pointsWon     = 3;
  newPrefs.pointsDraw    = 1;
  newPrefs.pointsLost    = 0;
  newPrefs.pointsPenalty = 0;
  newPrefs.tiebreaker[0] = tieBreakerPoints;
  newPrefs.tiebreaker[1] = tieBreakerGoalsDiff;
  newPrefs.tiebreaker[2] = tieBreakerGoals;
  newPrefs.tiebreaker[3] = tieBreakerMatchesWon;
  newPrefs.tiebreaker[4] = tieBreakerDirectCompare | tieBreakerNotSelected;
  newPrefs.tiebreaker[5] = tieBreakerPointsDiff    | tieBreakerNotSelected;
  newPrefs.tiebreaker[6] = tieBreakerPointsPercent | tieBreakerNotSelected;
  newPrefs.tiebreaker[7] = tieBreakerNone;
  for (i=0; i<5; i++) {
    newPrefs.colors[i].from  = 0;
    newPrefs.colors[i].to    = 0;
    newPrefs.colors[i].color = 255;
  }
  
  newPrefs.tableSettings[0] = newPrefs.tableSettings[5] = tableSettingsMatches;
  newPrefs.tableSettings[1] = tableSettingsGoals;
  newPrefs.tableSettings[2] = tableSettingsGoalsAgainst;
  newPrefs.tableSettings[3] = tableSettingsGoalsDiff;
  newPrefs.tableSettings[4] = newPrefs.tableSettings[9] = tableSettingsPts;
  newPrefs.tableSettings[6] = tableSettingsWon;
  newPrefs.tableSettings[7] = tableSettingsDraw;
  newPrefs.tableSettings[8] = tableSettingsLost;

  if ( name && (db = CreateDatabase(name)) ) {
    InitNewDatabase(db, &newPrefs);
    DmCloseDatabase(db);
  }
}

Boolean NewDBFormHandleEvent(EventPtr e)
{
  Boolean handled;
  
  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {

  case frmOpenEvent:
    InitFrmNewDB();
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    /* Ready */
    case ID_FrmNewDBButton:
      ReadFormNewDBValues();
      ChooseDB();

    /* Cancel */
    case ID_FrmNewDBButton+1:
      FrmReturnToForm(0);
      handled = true; 
    }
    break;

  default:
    break;
  }

  return handled;
}

/**
 * routines handling renaming a database name
 */
static void InitFrmRenameLeague()
{
  UInt16 i = LstGetSelection(FrmGetListPtr(ID_FrmLeague, ID_FrmLeagueList));
  if (i != noListSelection)
    SetField(ID_FrmRenameLeagueFld, 32, Items[i]);
}

static void RenameDatabaseFile()
{
  char* name = FldGetTextPtr(Id2Ptr(ID_FrmRenameLeagueFld));
  UInt16 i;
  if (name) {
    if ( noListSelection !=
         (i = LstGetSelection(FrmGetListPtr(ID_FrmLeague, ID_FrmLeagueList)) ) )
      RenameDatabase(i, name);
  }
}

Boolean RenameLeagueFormHandleEvent(EventPtr e)
{
  Boolean handled;

  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {

  case frmOpenEvent:
    InitFrmRenameLeague();
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    /* Ready */
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmRenameLeagueButton:
      RenameDatabaseFile();

    /* Cancel */
    case ID_FrmRenameLeagueButton+1:
      FrmReturnToForm(0);
      RedrawList(ID_FrmLeague, ID_FrmLeagueList);
      handled = true; 
      break;
    }
    break;

  default:
    break;
  }

  return handled;
}

/**
 * routines for handling League form;
 * update DBNo if entry in list was moved.
 */
static void updateDbNo(Int16 i, Int16 j)
{
  if (DBNo == i) { 
    DBNo = j;
  } else if (DBNo == j) {
    DBNo = i;
  }
}

Boolean LeagueFormHandleEvent(EventPtr e)
{
  Boolean handled;
  static ListPtr p;
  Int16 i;

  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {

  case frmOpenEvent:
    p = FrmGetListPtr(ID_FrmLeague, ID_FrmLeagueList);
    LstSetListChoices(p, Items, numOfItems);
    FrmDrawForm(FrmGetActiveForm());
    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmLeagueButton: /* Ready */
      handled = true; 
      FrmReturnToForm(0);
      FrmEraseForm(FrmGetActiveForm());
      DrawMatches(true);
      break;

    case ID_FrmLeagueButton+1: /* move entry up */
      i = LstGetSelection(p);
      if (i>0) {
	swapDbInfo(i,i-1);
	updateDbNo(i,i-1);
	// LstSetListChoices(p, Items, numOfItems);
	LstDrawList(p);
	LstSetSelection(p,i-1);
      }
      handled = true; 
      break;

    case ID_FrmLeagueButton+2: /* move entry down */
      i = LstGetSelection(p);
      if ( (i!=noListSelection) && (i<numOfItems-1) ) {
	swapDbInfo(i,i+1);
	updateDbNo(i,i+1);
	// LstSetListChoices(p, Items, numOfItems);
	LstDrawList(p);
	LstSetSelection(p,i+1);
      }
      handled = true; 
      break;

    case ID_FrmLeagueButton+3: /* Rename database name */
      FrmPopupForm(ID_FrmRenameLeague);
      handled = true;
      break;

    case ID_FrmLeagueButton+4: /* sort leagues alphabetically */
      SortDbInfo();
      LstDrawList(p);
      handled = true;

    }

  case penDownEvent:
    // this is the position of the (i)
    if (e->screenX >= 144 && e->screenX <= 155 &&
	e->screenY >= 0 && e->screenY <= 11) {
      // this way we don't have to use the default bold font.
      MyDoDialog(ID_FrmHelp, ID_FrmLeague, "Tipps", FrmHelpHandleEvent, InitFrmHelp);
      handled =true;
    }

  default:
    break;
  }

  return handled;
}

