/*
 * Bundesliga is Copyright (c) 1997-2003 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <PalmOS.h>

#include "bl.h"
#include "blrcp.h"

static void DrawFixtureItem(void*, Int16, Int16, RectangleType*) TBLFUNC;
static void InitFormShowFixture(FormType*, Int16*, Int16*) TBLFUNC;
static Int16 SortTableRecords(void* a, void* b, Int32 arg) TBLFUNC;
static void RedrawFormFixture(FormType*) TBLFUNC;
static void ResizeFormFixture(FormType*) TBLFUNC;

static TablePtr fixtureTable;
static Int16 tableSize;

/**
 * ShowFixture Form
 */

static void DrawFixtureItem(void* tableP, Int16 row, Int16 column, 
                            RectangleType *bounds)
{
  UInt16 n,x;
  Char buf[25], *p;
  DateTimeType dateP;
  UInt32 date;

  n = TblGetItemInt(tableP, row, column);
  date = TblGetRowData(tableP, row);
  x = bounds->topLeft.x;

  FntSetFont(0);

  switch (column) {

  // date of a match; stored as row data.
  case 0:
    StrCopy(buf,"");
    if (date>0) {
      TimSecondsToDateTime(date, &dateP);
      if (n) {
	// no need to show time, if it is "00:00"
	if (dateP.hour+dateP.minute>0) {
	  TimeToAscii(dateP.hour,dateP.minute,
		    PrefGetPreference(prefTimeFormat), buf);
	} else {
	  n = 0;
	}
      }
      if (!n) {
	DateToAscii(dateP.month,dateP.day,dateP.year,
		    PrefGetPreference(prefDateFormat), buf);
      }
    }
    break;

  // stats.team holds the team for which fixture is shown,
  // item data encodes the match as (opp|matchday|awayflag)
  // this way home/away is "stored".
  case 1:
    if (n & (1<<15))
      p = GetString(ID_tSTR_away);
    else 
      p = GetString(ID_tSTR_home);
    StrCopy(buf, p);
    StrCat(buf, theTeams[n%128-1]);
    break;

  // items 2 and 3 hold the result as (r1<<8|r2)
  case 2:
    if (n == 0xffff) {
      StrCopy(buf, "");
    } else {
      StrIToA(buf, n/256);
    }
    StrCat(buf," :");
    x += bounds->extent.x - FntCharsWidth(buf,StrLen(buf)) - 1;
    break;
  case 3:
    if (n == 0xffff) {
      StrCopy(buf, "");
    } else {
      StrIToA(buf, n%256);
    }
    break;
  }
  WinSetClip(bounds);
  WinDrawChars(buf, StrLen(buf), x, bounds->topLeft.y);
  WinResetClip();
}  

struct tableRecords {
  UInt32 date;
  UInt16 data1, data2;
};

// sort function used by SysInsertionSort()
static Int16 SortTableRecords(void* a, void* b, Int32 arg)
{
  if (((struct tableRecords*)a)->date < ((struct tableRecords*)b)->date)
    return -1;
  if (((struct tableRecords*)a)->date > ((struct tableRecords*)b)->date)
    return 1;
  return 0;
}

static void ResizeFormFixture(FormType* frm)
{
  Int16 x, y;
  Coord width;

  WinGetDisplayExtent( &x, &y );

  // Landscape Mode
  if (x > y) {
    tableSize = 10;
    width = TblGetColumnWidth(fixtureTable, 1);
    if (width < 120) {
      TblSetColumnWidth(fixtureTable, 1, width+54);
      width = TblGetColumnWidth(fixtureTable, 0);
      TblSetColumnWidth(fixtureTable, 0, width+10);
    }
  }

  // Portrait Mode
  if (x <= y) {
    tableSize = 16;
    width = TblGetColumnWidth(fixtureTable, 1);
    if (width >= 120) {
      TblSetColumnWidth(fixtureTable, 1, width-54);
      width = TblGetColumnWidth(fixtureTable, 0);
      TblSetColumnWidth(fixtureTable, 0, width-10);
    }
  }

  // Square Mode
  if (x == y) {
    tableSize = 10;
  }

  
}

static void RedrawFormFixture(FormType* frm)
{
  FrmDrawForm(frm);
  DrawButton(ID_FrmShowFixtureButton+0,ID_Bitmap+2);
  DrawButton(ID_FrmShowFixtureButton+1,ID_Bitmap+4);
  DrawButton(ID_FrmShowFixtureButton+2,ID_Bitmap+7);
  TblDrawTable(fixtureTable);
}

static void InitFormShowFixture(FormType* frm, Int16 *rows, Int16 *visible)
{
  Int16 col, i, j, row, vis;
  Int16 maxR1=99, maxR2=99;
  char *p, *q, buf[10];
  matchType *m;
  Int8 numMatches;
  DateTimeType dateP;
  Int16 width;
  UInt32 now;
  Boolean sortEntries = true;

  struct tableRecords* TableRecordsP;

  fixtureTable = Id2Ptr(ID_FrmShowFixtureTable);

  // set the table attributes
  for (row=0; row<TblGetNumberOfRows(fixtureTable); row++) {
    TblSetRowSelectable(fixtureTable, row, true);
    TblSetRowUsable(fixtureTable, row, false);
    for (col=0; col<4; col++) {
      TblSetItemStyle(fixtureTable, row, col, customTableItem);
      TblSetColumnUsable(fixtureTable, col, true);
      TblSetCustomDrawProcedure(fixtureTable, col, DrawFixtureItem);
    }
  }

  TableRecordsP = (struct tableRecords*)MemPtrNew(90*sizeof(struct tableRecords));

  // and now collect the table data.
  now = TimGetSeconds();
  now -= now % 86400;
  vis = 0;

  row = 0;
  for (i=0; i<prefs.NumMatchDays; i++) {

    p = q = MemHandleLock(DmQueryRecord(db,i+2));
    numMatches = (Int8)*p;
    p += 2;

    for (j=0; j<numMatches; j++) {
      m = (matchType*)p;
      if ( m->t1 == stats.team || m->t2 == stats.team ) {
	if ( m->t1 == stats.team ) {
	  if (m->r1 > maxR1) maxR1=m->r1;
	  if (m->r2 > maxR2) maxR2=m->r2;
	  TableRecordsP[row].data1 = m->t2 + (i<<7);
	  TableRecordsP[row].data2 = (m->r1<<8)+m->r2;
	} else {
	  if (m->r2 > maxR1) maxR1=m->r2;
	  if (m->r1 > maxR2) maxR2=m->r1;
	  TableRecordsP[row].data1 = m->t1 + (i<<7) + (1<<15);
	  TableRecordsP[row].data2 = (m->r2<<8)+m->r1;
	}
	TableRecordsP[row].date = m->Date;
	if (m->Date == 0)
	  sortEntries = false;
	row++;
      }
      p += sizeof(matchType);
    }
    MemPtrUnlock(q);
  }

  // sort the data by match date
  if (sortEntries)
    SysInsertionSort(TableRecordsP,row,
		     sizeof(struct tableRecords),SortTableRecords,0);

  // fill in the sorted data into table
  for (i=0; i<row; i++) {
    TblSetRowData(fixtureTable, i, TableRecordsP[i].date);
    TblSetItemInt(fixtureTable, i, 0, 0);
    TblSetItemInt(fixtureTable, i, 1, TableRecordsP[i].data1);
    TblSetItemInt(fixtureTable, i, 2, TableRecordsP[i].data2);
    TblSetItemInt(fixtureTable, i, 3, TableRecordsP[i].data2);
    if (TableRecordsP[i].date && TableRecordsP[i].date < now) 
      vis = i;
  }
  MemPtrFree(TableRecordsP);

  // from global settings compute maximum needed widths
  // of date and other columns.
  // the date used here is 2002/12/24 22:30 :-)
  TimSecondsToDateTime(3123613800UL, &dateP);
  DateToAscii(dateP.month,dateP.day,dateP.year,
	      PrefGetPreference(prefDateFormat), buf);
  width = FntCharsWidth(buf,StrLen(buf))+1;

  TblSetColumnWidth(fixtureTable,0,width);
  j = 150-width;

  StrIToA(buf,maxR1);
  StrCat(buf," :");
  width = FntCharsWidth(buf,StrLen(buf))+1;
  TblSetColumnWidth(fixtureTable,2,width);
  j -= width;

  StrIToA(buf,maxR2);
  width = FntCharsWidth(buf,StrLen(buf))+1;
  TblSetColumnWidth(fixtureTable,3,width);
  j -= width;

  TblSetColumnWidth(fixtureTable,1,j);

  RedrawFormFixture(frm);

  *rows = row;
  *visible = vis+1;
}

Boolean ShowFixtureFormHandleEvent(EventPtr e)
{
  Boolean handled;
  static Int16 from, rows;
  UInt16 row, column, val;
  static ScrollBarPtr scrollP;
  static ListPtr pLst;
  static FormType* frm;
  
  if ( (handled = ResizeHandleEvent(e)) )
    return true;

  switch (e->eType) {

  case frmOpenEvent:
    frm = FrmGetActiveForm();
    tableSize = 10;
    if (stats.team == 0) {
      if (prefs.favTeam)
	stats.team = prefs.favTeam;
      else
	stats.team = 1;
    }
    InitFormShowFixture(frm,&rows,&from);
    ResizeFormFixture(frm);
    from -= 5;
    if (from+tableSize > rows)
      from = rows-tableSize;
    if (from < 0)
      from = 0;

    pLst = Id2Ptr(ID_FrmShowFixtureList+1);
    LstSetListChoices(pLst, theTeams, prefs.NumTeams);
    LstSetHeight(pLst, prefs.NumTeams>10?10:prefs.NumTeams);
    LstSetSelection(pLst, stats.team-1);
    LstMakeItemVisible(pLst, stats.team-1);
    CtlSetLabel(Id2Ptr(ID_FrmShowFixtureList),theTeams[stats.team-1]);

    scrollP = Id2Ptr(ID_FrmShowFixtureScroll);
    RedrawTable(fixtureTable, scrollP, from, rows, 0, tableSize);

    handled = true; 
    break;

  case frmUpdateEvent:
    ResizeFormFixture(frm);
    RedrawFormFixture(frm);
    RedrawTable(fixtureTable, scrollP, from, rows, 0, tableSize);
    handled = true;
    break;

  // Scrollbar Event
  case sclRepeatEvent:
    from = e->data.sclRepeat.newValue;
    RedrawTable(fixtureTable, scrollP, from, rows, 0, tableSize);
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {

    // Switch Bar
    case ID_FrmShowFixtureButton:
      FrmGotoForm(ID_FrmBl);
      handled = true; 
      break;
    
    case ID_FrmShowFixtureButton+1:
      FrmGotoForm(ID_FrmTable);
      handled = true; 
      break;
    
    // date book button
    case ID_FrmShowFixtureButton+3: 
      if (DisplayConfirmationMessage(ID_ConfirmationMessages) == 0
	  && OpenDatebook()) {
	Int16 i, j;
	for (i=0; i<prefs.NumMatchDays; i++) {
	  char *p, *q;
	  Int8 numMatches;
	  matchType *m;
	  
	  p = q = MemHandleLock(DmQueryRecord(db,i+2));
	  numMatches = (Int8)*p;
	  p += 2;
	  
	  for (j=0; j<numMatches; j++) {
	    m = (matchType*)p;
	    if ( m->t1 == stats.team || m->t2 == stats.team ) {
	      MakeDatebookEntry(m);
	    }
	    p += sizeof(matchType);
	  }
	  MemPtrUnlock(q);
	}
	DmCloseDatabase(datebook);
      }	

      handled = true; 

    }
    break;

  case popSelectEvent:
    if (e->data.popSelect.selection+1 != stats.team) {
      stats.team = e->data.popSelect.selection+1;
      FrmGotoForm(ID_FrmShowFixture);
    }
    handled = true;

  case keyDownEvent:
    if (e->data.keyDown.chr == vchrPageUp) {
      if (from>0) {
	from -= tableSize;
	if (from < 0) from = 0;
	RedrawTable(fixtureTable, scrollP, from, rows, 0, tableSize);
      }
      handled = true; 
    }

    if (e->data.keyDown.chr == vchrPageDown) {
      if (from+tableSize < rows) {
	from += tableSize;
	if (from+tableSize>rows) from = rows - tableSize;
	RedrawTable(fixtureTable, scrollP, from, rows, 0, tableSize);
      }
      handled = true; 
    }

    if (e->data.keyDown.chr == vchrRockerLeft || 
	e->data.keyDown.keyCode == fiveWayLeft) {
      FrmGotoForm(ID_FrmTable);
      handled = true;
    }

    if (e->data.keyDown.chr == vchrRockerRight || 
	e->data.keyDown.keyCode == fiveWayRight) {
      FrmGotoForm(ID_FrmBl);
      handled = true;
    }

    if (e->data.keyDown.chr == vchrRockerCenter || 
        (e->data.keyDown.keyCode == fiveWaySelect &&
         (e->data.keyDown.modifiers & autoRepeatKeyMask) == 0) ) {
      EventType evt;
      evt.eType = ctlSelectEvent;
      evt.data.ctlSelect.controlID = ID_FrmShowFixtureList;
      EvtAddEventToQueue(&evt);
      handled = true;
    }

    break;

  // show fixture of another team
  case tblSelectEvent:
    TblGetSelection(fixtureTable, &row, &column);
    TblUnhighlightSelection(fixtureTable);
    switch(column) {
    case 0:
      TblSetItemInt(fixtureTable,row,0,!TblGetItemInt(fixtureTable, row, 0));
      TblMarkRowInvalid(fixtureTable, row);
      TblRedrawTable(fixtureTable);
      break;

    case 1:
      val = (TblGetItemInt(fixtureTable, row, 1)>>7) & 127;
      matchDay = val + 1;
      FrmGotoForm(ID_FrmBl);
    }
    handled = true;
    break;

  default:
    break;
  }

  return handled;
}
