/*
 * Bundesliga is Copyright (c) 1997-2003 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <PalmOS.h>

#include "bl.h"
#include "blrcp.h"

static TablePtr handicapTable;
static Int16 handicapTableRows;

static void DrawHandicapItem(void*, Int16, Int16, RectangleType*) EDITFUNC;
static void InitFormHandicap(void) EDITFUNC;
static Boolean GetHandicap(handicapType *h, Int8 i) EDITFUNC;
static Int16 numberOfHandicapEntries(Int16 spTag) EDITFUNC;
static void GetNextHandicap(handicapType *h, Int16 spTag) EDITFUNC;
static void SaveHandicaps(void) EDITFUNC;

Boolean HandicapEditFormHandleEvent(EventPtr e)
{
  Boolean handled;
  static UInt16 row, column;
  static ListPtr p;
  Int16 val, i, j;
  UInt16 sel;
  char buf[4], *cval;

  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {
    
  case frmOpenEvent:
    TblGetSelection(handicapTable, &row, &column);

    // match day
    val = TblGetItemInt(handicapTable, row, 1);
    StrIToA(buf, val+1);
    SetField(ID_FrmHandicapEditFld,4,buf);

    // value
    val = TblGetItemInt(handicapTable, row, 2);
    StrIToA(buf, val);
    /* check wether point or goal handicap and set the corresponding field */
    SetField(ID_FrmHandicapEditFld+TblGetItemInt(handicapTable,row,3)+1,
	     4,buf);

    val = TblGetItemInt(handicapTable, row, 0);
    if (val>0) {
      FrmCopyLabel(FrmGetActiveForm(), 
		   ID_FrmHandicapEditPopup, 
		   theTeams[val-1]);
    }

    FrmDrawForm(FrmGetActiveForm());

    p = Id2Ptr(ID_FrmHandicapEditPopup+1);
    LstSetListChoices(p, theTeams, prefs.NumTeams);
    LstSetHeight(p, prefs.NumTeams>10?10:prefs.NumTeams);
    if (val>0)
      LstSetSelection(p, val-1);

    handled = true; 
    break;
    
  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmHandicapEditButton: /* Ready */
      /*
       * read in fields and save changes into table
       */
      sel = LstGetSelection(p);
      if (sel != noListSelection) {
	TblSetItemInt(handicapTable, row, 0, sel+1);
      }
      cval = FldGetTextPtr(Id2Ptr(ID_FrmHandicapEditFld));
      if (cval && StrLen(cval)) {
	val = StrAToI(cval);
	if (val>0 && val<=prefs.NumMatchDays) {
	  TblSetItemInt(handicapTable, row, 1, val-1);
	}
      }
      cval = FldGetTextPtr(Id2Ptr(ID_FrmHandicapEditFld+1));
      if (cval && StrLen(cval)) {
	val = StrAToI(cval);
	if (val!=0) {
	  TblSetItemInt(handicapTable, row, 2, val);
	  TblSetItemInt(handicapTable, row, 3, 0);
	}
      } else {
	cval = FldGetTextPtr(Id2Ptr(ID_FrmHandicapEditFld+2));
	if (cval && StrLen(cval)) {
	  val = StrAToI(cval);
	  if (val!=0) {
	    TblSetItemInt(handicapTable, row, 2, val);
	    TblSetItemInt(handicapTable, row, 3, 1);
	  }
	}
      }
      goto FrmHandicapLabel;

    case ID_FrmHandicapEditButton+2: /* Delete */
      /*
       * delete handicap entry from table
       */
      handicapTableRows--;
      for (i=row; i<handicapTableRows; i++) {
	for (j=0; j<4; j++) {
	  TblSetItemInt(handicapTable, i, j, 
			TblGetItemInt(handicapTable, i+1, j));
	}
      }
      TblSetRowUsable(handicapTable,handicapTableRows,false);
      
    case ID_FrmHandicapEditButton+1: /* Cancel */
    FrmHandicapLabel:
      handled = true; 
      FrmReturnToForm(0);
      TblDrawTable(handicapTable);
      break;
      
    }
    break;

  case penDownEvent:
    // this is the position of the (i)
    if (e->screenX >= 144 && e->screenX <= 155 &&
	e->screenY >= 0 && e->screenY <= 11) {
      // this way we don't have to use the default bold font.
      MyDoDialog(ID_FrmHelp, ID_FrmHandicap, "Tipps", FrmHelpHandleEvent, InitFrmHelp);
      handled =true;
    }

  default:
    break;
  }
  
  return handled;
}

static void DrawHandicapItem(void *tableP, Int16 row, Int16 column, 
			     RectangleType *bounds)
{
  char buf[25], *r;
  Int16 n = TblGetItemInt(tableP, row, column);

  FntSetFont(0);

  switch (column) {
  case 0:
    if (n>0) {
      StrCopy(buf, theTeams[n-1]);
    } else 
      StrCopy(buf,"-----");
    break;
  case 1:
    n+=1;
  case 2:
    StrIToA(buf,n);
    break;
  case 3:
    StrCopy(buf,r = GetString(ID_tSTR_HandicapType+n));
    break;
  }
  WinEraseRectangle(bounds, 0);
  WinDrawChars(buf, StrLen(buf), bounds->topLeft.x, bounds->topLeft.y);
}

/*
 * initialize table variables to default values;
 */
static void InitFormHandicap()
{
  Int16 i, j;

  handicapTable = Id2Ptr(ID_FrmHandicapTable);
  handicapTableRows = 0;

  for (i=0; i<TblGetNumberOfRows(handicapTable); i++) {
    TblSetRowSelectable(handicapTable, i, true);
    TblSetRowUsable(handicapTable, i, false);
    for (j=0; j<4; j++) {
      TblSetItemStyle(handicapTable, i, j, customTableItem);
      TblSetColumnUsable(handicapTable, j, true);
      TblSetCustomDrawProcedure(handicapTable, j, DrawHandicapItem);
    }
  }

}

/*
 * Callback function, that is called from LoopRecords().
 * Set TableItems according to handicap entry in database.
 */
static Boolean GetHandicap(handicapType *h, Int8 i)
{
  if (handicapTableRows<TblGetNumberOfRows(handicapTable)) {
    if (handicapTableRows<10)
      TblSetRowUsable(handicapTable, handicapTableRows, true);
    TblSetItemInt(handicapTable, handicapTableRows, 0, h->team);
    TblSetItemInt(handicapTable, handicapTableRows, 1, i);
    TblSetItemInt(handicapTable, handicapTableRows, 2, h->val);
    TblSetItemInt(handicapTable, handicapTableRows, 3, h->type);
    handicapTableRows++;
  }
  return true;
}

/*
 * Used for saving the handicaps; returns number of
 * entries for given match day.
 */
static Int16 numberOfHandicapEntries(Int16 matchday)
{
  Int16 i, num = 0;
  for (i=0; i<handicapTableRows; i++) {
    if ( TblGetItemInt(handicapTable, i, 1) == matchday )
      num++;
  }
  return num;
}

/*
 * Used for saving the handicaps; fills in given structure
 * h with values from next entry for given match day.
 * Removes used line from Table.
 */
static void GetNextHandicap(handicapType *h, Int16 matchday)
{
  Int16 i, j, k;

  for (k=0; k<handicapTableRows; k++) {
    if ( TblGetItemInt(handicapTable, k, 1) == matchday ) {
      h->team = TblGetItemInt(handicapTable, k, 0);
      h->val  = TblGetItemInt(handicapTable, k, 2);
      h->type = TblGetItemInt(handicapTable, k, 3);
      break;
    }
  }

  handicapTableRows--;
  for (i=k; i<handicapTableRows; i++) {
    for (j=0; j<4; j++) {
      TblSetItemInt(handicapTable, i, j, 
		    TblGetItemInt(handicapTable, i+1, j));
    }
  }
  
}

/*
 * Loops over all records and resizes them according number of
 * handicaps for each match day.
 */
static void SaveHandicaps()
{
  Int16 i, j;
  Int8 numHandicaps, numTeams;
  char *p;
  handicapType h;
  UInt32 size, offset;

  if (prefs.options & readonly)
    return;

  for (i=0; i<prefs.NumMatchDays; i++) {
    p = MemHandleLock(DmQueryRecord(db,i+2));
    numTeams = (Int8)*p;
    MemPtrUnlock(p);
    numHandicaps = numberOfHandicapEntries(i);

    size = 2 + numTeams*sizeof(matchType) + numHandicaps*sizeof(handicapType);
    p = MemHandleLock(DmResizeRecord(db,i+2,size));

    DmWrite(p,0,&numTeams,1);
    DmWrite(p,1,&numHandicaps,1);

    offset = 2 + numTeams*sizeof(matchType);
    
    for (j=0; j<numHandicaps; j++) {
      GetNextHandicap(&h, i);
      DmWrite(p,offset,&h,sizeof(handicapType));
      offset += sizeof(handicapType);
    }

    MemPtrUnlock(p);
    DmReleaseRecord(db, i+2, true);
  }

}

Boolean HandicapFormHandleEvent(EventPtr e)
{
  Boolean handled;
  handicapType h;
  static char from;
  static ScrollBarPtr scrollP;

  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {
    
  case frmOpenEvent:
    /*
     * loop over all records, read in handicaps and show them
     * in a table.
     */
    InitFormHandicap();
    scrollP = Id2Ptr(ID_FrmHandicapScroll);
    stats.evalHandicap = true;
    LoopRecords(NULL,GetHandicap,0xffffffff);
    FrmDrawForm(FrmGetActiveForm());
    from = 0;
    handled = true; 
    break;
    
  // Scrollbar Event
  case sclRepeatEvent:
    from = e->data.sclRepeat.newValue;
    RedrawTable(handicapTable, scrollP, from, handicapTableRows, 0, 10);
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmHandicapButton: /* Ready */
      SaveHandicaps();
      handled = true; 
      FrmReturnToForm(0);
      break;
    
    case ID_FrmHandicapButton+1: /* New */
      /*
       * add a new entry to the table.
       */
      h.team = h.val = h.type = 0;
      GetHandicap(&h,0);
      TblDrawTable(handicapTable);
      handled = true; 
      break;
    }

  case keyDownEvent:
    switch (e->data.keyDown.chr) {
    case pageUpChr:
      if (from > 0) {
	from -= 10;
	if (from < 0) 
	  from = 0;
        RedrawTable(handicapTable, scrollP, from, handicapTableRows, 0, 10);
      }
      handled = true; 
      break;

    case pageDownChr:
      if (from+10 < handicapTableRows) {
	from += 10;
	if (from+10>handicapTableRows) 
	  from = handicapTableRows - 10;
        RedrawTable(handicapTable, scrollP, from, handicapTableRows, 0, 10);
      }
      handled = true; 
      break;
    }
    break;

  case tblSelectEvent:
    FrmPopupForm(ID_FrmHandicapEdit);
    handled = true; 
    break;

  case penDownEvent:
    // this is the position of the (i)
    if (e->screenX >= 144 && e->screenX <= 155 &&
	e->screenY >= 0 && e->screenY <= 11) {
      // this way we don't have to use the default bold font.
      MyDoDialog(ID_FrmHelp, ID_FrmHandicapEdit, "Tipps", FrmHelpHandleEvent, InitFrmHelp);
      handled =true;
    }

  default:
    break;
  }
  
  return handled;
}
