/*
 * Bundesliga is Copyright (c) 1997-2003 by Thomas Pundt
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose, without fee, and without a written agreement is hereby granted,
 * provided that the above copyright notice and this paragraph and the
 * following two paragraphs appear in all copies.
 *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
 * SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
 * THE AUTHOR HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHOR SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS"
 * BASIS, AND THE AUTHOR HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

#include <PalmOS.h>

#include "bl.h"
#include "blrcp.h"

static TablePtr DatabaseSettingsInitTable(void) TBLFUNC;
static void DrawDBSTableItem(void*,Int16,Int16,RectangleType*) TBLFUNC;
static void DBSTableSwapRows(const TablePtr,Int16,Int16,Boolean) TBLFUNC;
static void FrmDrawButtonBackGround(Int16,UInt8) TBLFUNC;

void FrmDrawButtonBackGround(Int16 base, UInt8 color)
{
  RGBColorType rgb, oldrgb;

  if (!FeatureSet35)
    return;

  UIColorGetTableEntryRGB(UIObjectFill,&oldrgb);
  rgb = oldrgb;
  if (color != 255) {
    WinIndexToRGB(color,&rgb);
  }
  UIColorSetTableEntry(UIObjectFill,&rgb);
  CtlDrawControl(Id2Ptr(base));
  UIColorSetTableEntry(UIObjectFill,&oldrgb);
}

/**
 * routine that handles the settings dialog
 */
Boolean SettingsFormHandleEvent(EventPtr e)
{
  Boolean handled;
  FormPtr frm;
  static FieldPtr f;
  MemHandle h;
  static UInt8 options;
  char *alarm;
  IndexedColorType color;
  ListPtr pLst;
  Int16 i;

  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {

  case frmOpenEvent:
    /* read preferences and set form according to these */
    frm = FrmGetActiveForm();
    options = appPrefs.options;
    if ( (options & (editMode|dontSaveEM)) == (editMode|dontSaveEM) ) 
      i = 2;
    else if (options & editMode) 
      i = 1;
    else
      i = 0;

    pLst = Id2Ptr(ID_FrmSettings+3);
    LstSetSelection(pLst,i);
    CtlSetLabel(Id2Ptr(ID_FrmSettings+2), LstGetSelectionText(pLst,i));

    pLst = Id2Ptr(ID_FrmSettings+5);
    LstSetSelection(pLst,matchSymbol);
    CtlSetLabel(Id2Ptr(ID_FrmSettings+4), LstGetSelectionText(pLst,matchSymbol));

    FrmSetControlValue(frm, FrmGetObjectIndex(frm, ID_FrmSettingsAlarm), 
		       options & setAlarm);
    f = Id2Ptr(ID_FrmSettingsAlarm+1);
    if (!(h = FldGetTextHandle(f))) {
      h = MemHandleNew(4);
    }
    StrIToA(MemHandleLock(h), appPrefs.minutesAlarm);
    MemHandleUnlock(h);
    FldSetTextHandle(f, h);

    if (!FeatureSet35) {
      for (i=0; i<6; i++) {
	CtlSetUsable(Id2Ptr(ID_FrmSettings+30+i), false);
	FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmSettings+30+i));
      }
    }

    FrmDrawForm(frm);

    for (i=0; i<5; i++)
      FrmDrawButtonBackGround(ID_FrmSettings+30+i,appPrefs.color[i]);

    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmSettings: /* Ready => save settings */
      appPrefs.options = options; 
      alarm = FldGetTextPtr(f);
      if (alarm && StrLen(alarm)) {
	appPrefs.minutesAlarm = StrAToI(alarm);
      }
      matchSymbol = (options & 24) >> 3;

    case ID_FrmSettings+1: /* Cancel */
      FrmGotoForm(ID_FrmBl);
      handled = true; 
      break;

    case ID_FrmSettingsAlarm:  /* checkbox "Date Book Alarm" */
      options ^= setAlarm;
      handled = true;
      break;

    case ID_FrmSettings+35:  /* Reset button */
      for (i=0; i<5; i++) {
	appPrefs.color[i] = 255;
	FrmDrawButtonBackGround(ID_FrmSettings+30+i,255);
      }
      handled = true;
      break;

    default:
      if (FeatureSet35
          && e->data.ctlSelect.controlID >= ID_FrmSettings+30
	  && e->data.ctlSelect.controlID <= ID_FrmSettings+34) {
	handled = true; 
	i = e->data.ctlSelect.controlID - ID_FrmSettings - 30;
	color = appPrefs.color[i];
	if (UIPickColor(&color,0,UIPickColorStartPalette,0,0)) {
	  appPrefs.color[i] = color;
	}
	FrmDrawButtonBackGround(e->data.ctlSelect.controlID,color);
      }
    }
    break;

  case popSelectEvent:
    if (e->data.popSelect.controlID == ID_FrmSettings+2) {
      switch (e->data.popSelect.selection) {
      case 0:
	options &= (0xff ^ (editMode|dontSaveEM));
	break;
      case 1:
	options |= editMode;
	options &= (0xff ^ dontSaveEM);
        break;
      case 2:
	options |= (editMode|dontSaveEM);
      }
    }

    if (e->data.popSelect.controlID == ID_FrmSettings+4) {
      options &= (0xff ^ (8|16));
      options |= (e->data.popSelect.selection << 3);
    }

  case penDownEvent:
    // this is the position of the (i)
    if (e->screenX >= 144 && e->screenX <= 155 &&
	e->screenY >= 0 && e->screenY <= 11) {
      // this way we don't have to use the default bold font.
      MyDoDialog(ID_FrmHelp, ID_FrmSettings, "Tipps", FrmHelpHandleEvent, InitFrmHelp);
      handled =true;
    }

  default:
    break;
  }

  return handled;
}

/**
 * routine that handles the settings dialog
 */
static void DBSTableSwapRows(const TablePtr tPtr, Int16 row1, Int16 row2, Boolean large)
{
  Int16 item1, item2, i;

  for (i=0; i<2; i++) {
    item1 = TblGetItemInt(tPtr, row1, i);
    item2 = TblGetItemInt(tPtr, row2, i);
    TblSetItemInt(tPtr, row1, i, item2);
    TblSetItemInt(tPtr, row2, i, item1);
  }

  i = row1;
  if (i>2) i=2;
  if (large) i=0;
  RedrawTable(tPtr,Id2Ptr(ID_FrmDatabaseSettingsScroll),i,7,0,large?7:5);
  TblSelectItem(tPtr,row1,0);

}

static void DrawDBSTableItem(void* tableP, Int16 row, Int16 column, 
			     RectangleType *bounds)
{
  int item = TblGetItemInt(tableP, row, column);
  char *p;

  if (item < 0 || item > 7)
    return;

  p = GetString(ID_tSTR_TieBreaker+item);
  WinDrawChars(p, StrLen(p), bounds->topLeft.x, bounds->topLeft.y);
}

static TablePtr DatabaseSettingsInitTable(void)
{
  TablePtr tPtr = Id2Ptr(ID_FrmDatabaseSettings+30);
  Int16 i, numRows;

  numRows = TblGetNumberOfRows(tPtr);
  for (i=0; i<numRows; i++) {
    TblSetItemStyle(tPtr, i, 0,customTableItem );
    TblSetItemInt(tPtr, i, 0, prefs.tiebreaker[i]&127);
    TblSetItemStyle(tPtr, i, 1, checkboxTableItem);
    TblSetItemInt(tPtr, i, 1, !(prefs.tiebreaker[i] >> 7) );
    TblSetRowUsable(tPtr, i, false);
  }

  TblSetCustomDrawProcedure(tPtr, 0, DrawDBSTableItem);
  TblSetColumnUsable(tPtr,0,true);
  TblSetColumnUsable(tPtr,1,true);
  return tPtr;
}

Boolean DatabaseSettingsFormHandleEvent(EventPtr e)
{
  Boolean handled;
  static FormPtr frm;
  static FieldPtr f;
  MemHandle h;
  static Int8 leagueOptions;
  char *val;
  static TablePtr tPtr;
  Int16 i, j, numRows;
  static Int16 x, y;
  RectangleType bounds;
  static ScrollBarType *scrollP;

  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {

  case frmOpenEvent:
    /* read preferences and set form according to these */
    frm = FrmGetActiveForm();

    scrollP = Id2Ptr(ID_FrmDatabaseSettingsScroll);

    leagueOptions = prefs.options;
    FrmSetControlValue(frm, FrmGetObjectIndex(frm, ID_FrmDatabaseSettings+10), 
		       leagueOptions & readonly);
    FrmSetControlValue(frm, FrmGetObjectIndex(frm, ID_FrmDatabaseSettings+11),
                       leagueOptions & needWideResultField);
      

    f = Id2Ptr(ID_FrmDatabaseSettings+20);
    if (!(h = FldGetTextHandle(f))) {
      h = MemHandleNew(4);
    }
    StrIToA(MemHandleLock(h), prefs.pointsWon);
    MemHandleUnlock(h);
    FldSetTextHandle(f, h);

    f = Id2Ptr(ID_FrmDatabaseSettings+21);
    if (!(h = FldGetTextHandle(f))) {
      h = MemHandleNew(4);
    }
    StrIToA(MemHandleLock(h), prefs.pointsDraw);
    MemHandleUnlock(h);
    FldSetTextHandle(f, h);

    f = Id2Ptr(ID_FrmDatabaseSettings+22);
    if (!(h = FldGetTextHandle(f))) {
      h = MemHandleNew(4);
    }
    StrIToA(MemHandleLock(h), prefs.pointsLost);
    MemHandleUnlock(h);
    FldSetTextHandle(f, h);

    f = Id2Ptr(ID_FrmDatabaseSettings+23);
    if (!(h = FldGetTextHandle(f))) {
      h = MemHandleNew(4);
    }
    StrIToA(MemHandleLock(h), prefs.pointsPenalty);
    MemHandleUnlock(h);
    FldSetTextHandle(f, h);

    tPtr = DatabaseSettingsInitTable();

  case frmUpdateEvent:
    FrmDrawForm(frm);

    TblGetBounds(tPtr, &bounds);
    bounds.topLeft.x -= 2;
    bounds.topLeft.y -= 1;
    bounds.extent.x  += 4;
    bounds.extent.y  += 2;
    WinDrawRectangleFrame(simpleFrame,&bounds);

    WinGetDisplayExtent( &x, &y );
    if (x<y) { // portrait mode => more rows in table visible
      RedrawTable(tPtr, scrollP, 0, 7, 0, 7);
    } else {
      RedrawTable(tPtr, scrollP, 0, 7, 0, 5);
    }
    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmDatabaseSettings: /* Ready => save settings */
      prefs.options = leagueOptions;
      val = FldGetTextPtr(Id2Ptr(ID_FrmDatabaseSettings+20));
      if (val && StrLen(val)) {
	prefs.pointsWon = StrAToI(val);
      }
      val = FldGetTextPtr(Id2Ptr(ID_FrmDatabaseSettings+21));
      if (val && StrLen(val)) {
	prefs.pointsDraw = StrAToI(val);
      }
      val = FldGetTextPtr(Id2Ptr(ID_FrmDatabaseSettings+22));
      if (val && StrLen(val)) {
	prefs.pointsLost = StrAToI(val);
      }
      val = FldGetTextPtr(Id2Ptr(ID_FrmDatabaseSettings+23));
      if (val && StrLen(val)) {
	prefs.pointsPenalty = StrAToI(val);
      }

      // store updates made in tiebreaker table
      tPtr = Id2Ptr(ID_FrmDatabaseSettings+30);
      numRows = TblGetNumberOfRows(tPtr);
      for (i=0; i<numRows; i++) {
	int method, checked;
	method = TblGetItemInt(tPtr, i, 0);
	checked = TblGetItemInt(tPtr, i, 1);
	prefs.tiebreaker[i] = method | (checked?0:128);
      }

    case ID_FrmDatabaseSettings+1: /* Cancel */
      FrmGotoForm(ID_FrmBl);
      handled = true; 
      break;

    case ID_FrmDatabaseSettings+2: /* shift tiebreaker up */
      tPtr = Id2Ptr(ID_FrmDatabaseSettings+30);
      if ( TblGetSelection(tPtr, &i, &j) ) {
	if (i>0) 
	  DBSTableSwapRows(tPtr,i-1,i,x<y);
      }
      handled = true; 
      break;

    case ID_FrmDatabaseSettings+3: /* ... or down */
      tPtr = Id2Ptr(ID_FrmDatabaseSettings+30);
      if ( TblGetSelection(tPtr, &i, &j) ) {
	if (i<6) 
	  DBSTableSwapRows(tPtr,i+1,i,x<y);
      }
      handled = true; 
      break;

    case ID_FrmDatabaseSettings+4: /* define table colors */
      handled = true; 
      FrmPopupForm(ID_FrmDatabaseColors);
      break;

    case ID_FrmDatabaseSettings+10: /* read-only option */
      leagueOptions ^= readonly;
      handled = true;
      break;

    case ID_FrmDatabaseSettings+11: /* need wide fields */
      leagueOptions ^= needWideResultField;
      handled = true;
      break;

    }
    break;

  // Scrollbar Event
  case sclRepeatEvent:
    RedrawTable(tPtr, scrollP, e->data.sclRepeat.newValue, 
		7, 0, 5);
    break;
  
  case penDownEvent:
    // this is the position of the (i)
    if (e->screenX >= 144 && e->screenX <= 155 &&
	e->screenY >= 0 && e->screenY <= 11) {
      // this way we don't have to use the default bold font.
      MyDoDialog(ID_FrmHelp, ID_FrmDatabaseSettings, "Tipps", FrmHelpHandleEvent, InitFrmHelp);
      handled =true;
    }

  default:
    break;
  }

  return handled;
}

Boolean DatabaseColorsFormHandleEvent(EventPtr e)
{
  Boolean handled;
  FormPtr frm;
  static FieldPtr f;
  MemHandle h;
  IndexedColorType color;
  Int16 i, j;
  char *val;

  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {

  case frmOpenEvent:
    /* read preferences and set form according to these */
    frm = FrmGetActiveForm();

    for (i=0; i<5; i++) {
      f = Id2Ptr(ID_FrmDatabaseColors+i*2+10);
      if (!(h = FldGetTextHandle(f))) {
	h = MemHandleNew(4);
      }
      StrIToA(MemHandleLock(h), prefs.colors[i].from);
      MemHandleUnlock(h);
      FldSetTextHandle(f, h);

      f = Id2Ptr(ID_FrmDatabaseColors+i*2+11);
      if (!(h = FldGetTextHandle(f))) {
	h = MemHandleNew(4);
      }
      StrIToA(MemHandleLock(h), prefs.colors[i].to);
      MemHandleUnlock(h);
      FldSetTextHandle(f, h);

      if (!FeatureSet35) {
	CtlSetUsable(Id2Ptr(ID_FrmDatabaseColors+30+i), false);
	FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmDatabaseColors+30+i));
      }

    }

    if (!FeatureSet35) {
      CtlSetUsable(Id2Ptr(ID_FrmDatabaseColors+35), false);
      FrmHideObject(frm,FrmGetObjectIndex(frm,ID_FrmDatabaseColors+35));
    }

    FrmDrawForm(frm);
    for (i=0; i<5; i++)
      FrmDrawButtonBackGround(ID_FrmDatabaseColors+30+i,prefs.colors[i].color);

    handled = true; 
    break;

  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {
    case ID_FrmDatabaseColors: /* Ready => save settings */
      for (i=0; i<5; i++) {
	val = FldGetTextPtr(Id2Ptr(ID_FrmDatabaseColors+i*2+10));
	if (val && StrLen(val)) {
	  j = StrAToI(val);
	  prefs.colors[i].from = j;
	}

	val = FldGetTextPtr(Id2Ptr(ID_FrmDatabaseColors+i*2+11));
	if (val && StrLen(val)) {
	  j = StrAToI(val);
	  prefs.colors[i].to = j;
	}
      }

    case ID_FrmDatabaseColors+1: /* Cancel */
      FrmReturnToForm(0);
      handled = true; 
      break;

    case ID_FrmDatabaseColors+35: /* Reset colors */
      for (i=0; i<5; i++) {
	prefs.colors[i].color = 255;
	FrmDrawButtonBackGround(ID_FrmDatabaseColors+30+i,255);
      }
      handled = true; 
      break;

    default:
      if (e->data.ctlSelect.controlID >= ID_FrmDatabaseColors+30
	  && e->data.ctlSelect.controlID <= ID_FrmDatabaseColors+34) {
	handled = true; 
	i = e->data.ctlSelect.controlID - ID_FrmDatabaseColors - 30;
	color = prefs.colors[i].color;
	if (UIPickColor(&color,0,UIPickColorStartPalette,0,0)) {
	  prefs.colors[i].color = color;
	}
	FrmDrawButtonBackGround(e->data.ctlSelect.controlID,color);
      }
    }
    break;

  case penDownEvent:
    // this is the position of the (i)
    if (e->screenX >= 144 && e->screenX <= 155 &&
	e->screenY >= 0 && e->screenY <= 11) {
      // this way we don't have to use the default bold font.
      MyDoDialog(ID_FrmHelp, ID_FrmDatabaseColors, "Tipps", FrmHelpHandleEvent, InitFrmHelp);
      handled =true;
    }

  default:
    break;
  }

  return handled;
}

