#include <PalmOS.h>

#include "bl.h"
#include "blrcp.h"

static TablePtr leagueTable;
static UInt32 computePartition(void) TBLFUNC;
static Int16 cmpByPosition(void*, void*, Int32) TBLFUNC;
static char cmpTeamsByTiebreaker(TeamResultType *a, TeamResultType *b, UInt8) TBLFUNC;
static void AddTeamResult(Int8 n, UInt8 r1, UInt8 r2, UInt16 penalty, Boolean home) TBLFUNC;
static Boolean EvalResult(matchType *m, Int8 matchday) TBLFUNC;
static Boolean EvalHandicap(handicapType *h, Int8 i) TBLFUNC;
static void DrawTableItem(void*, Int16, Int16, RectangleType*) TBLFUNC;
static void RedrawFormTable(FormType*) TBLFUNC;
static void InitFormTable(void) TBLFUNC;
static void InitTable(FormType*) TBLFUNC;
static Int16 GetWidthOfString(Int16) TBLFUNC;
static void ResizeTableColumns(void) TBLFUNC;
static Boolean SetTableFilter(Int16 selection) TBLFUNC; 

Int16 lastSelection;
static Int16 tableView;
static Int16 tableSize;
static Boolean landscape;

static struct {
  Int16 goals, agoals, diff;
} maxValues;

/**
 * general Compute Table functions
 */

Boolean isHomeMatch(matchType *m, Int8 team, Int8 matchDay)
{
  return team == m->t1;
}

Boolean isAwayMatch(matchType *m, Int8 team, Int8 matchDay)
{
  return team == m->t2;
}

Boolean inBetween(matchType *m, Int8 team, Int8 matchDay)
{
  return (matchDay>=stats.first && matchDay<=stats.last);
}

static Int16 cmpByPosition(void *a, void *b, Int32 other)
{
  if (((TeamResultType*)a)->Position > ((TeamResultType*)b)->Position) return 1;
  if (((TeamResultType*)a)->Position < ((TeamResultType*)b)->Position) return -1;
  return 0;
}

static char cmpTeamsByTiebreaker(TeamResultType *a, TeamResultType *b, UInt8 t)
{
  switch (t) {
  case tieBreakerPoints:
    if (a->Points > b->Points) return 1;
    if (a->Points < b->Points) return -1;
    break;
  case tieBreakerGoalsDiff:
    if (a->Goals + b->AGoals > b->Goals + a->AGoals) return 1;
    if (a->Goals + b->AGoals < b->Goals + a->AGoals) return -1;
    break;
  case tieBreakerPointsDiff:
    if (a->Points + b->APoints > b->Points + a->APoints) return 1;
    if (a->Points + b->APoints < b->Points + a->APoints) return -1;
    break;
  case tieBreakerGoals:
    if (a->Goals > b->Goals) return 1;
    if (a->Goals < b->Goals) return -1;
    break;
  case tieBreakerMatchesWon:
    if (a->Won > b->Won) return 1;
    if (a->Won < b->Won) return -1;
    break;
  }
  return 0;
}

static void AddTeamResult(Int8 n, UInt8 r1, UInt8 r2, UInt16 penalty, Boolean home)
{
  Int8 pointsWon  = prefs.pointsWon;  
  Int8 pointsDraw = prefs.pointsDraw;  
  Int8 pointsLost = prefs.pointsLost;
  if (prefs.pointsPenalty && penalty) {
    pointsWon  = prefs.pointsDraw+prefs.pointsPenalty;
    pointsLost = prefs.pointsDraw;
  }

  if (n>0) n--;

  TeamResult[n].Goals  += r1;
  TeamResult[n].AGoals += r2;
  TeamResult[n].Matches++;
  if (r1>r2) {
    TeamResult[n].Points+=pointsWon;
    TeamResult[n].APoints+=pointsLost;
    if (prefs.pointsPenalty && penalty)
      TeamResult[n].WonPen++;
    else
      TeamResult[n].Won++;
  } else if (r1==r2) {
    TeamResult[n].Points+=pointsDraw;
    TeamResult[n].APoints+=pointsDraw;
    TeamResult[n].Rem++;
  } else {
    TeamResult[n].Points+=pointsLost;
    TeamResult[n].APoints+=pointsWon;
    if (prefs.pointsPenalty && penalty) 
      TeamResult[n].LostPen++;
    else
      TeamResult[n].Lost++;
  }

  if (home) {
    TeamResult[n].HGoals  += r1;
    TeamResult[n].HAGoals += r2;
    TeamResult[n].HMatches++;
    if (r1>r2) {
      TeamResult[n].HPoints+=pointsWon;
      TeamResult[n].HAPoints+=pointsLost;
      if (prefs.pointsPenalty && penalty)
	TeamResult[n].HWonPen++;
      else
	TeamResult[n].HWon++;
    } else if (r1==r2) {
      TeamResult[n].HPoints+=pointsDraw;
      TeamResult[n].HAPoints+=pointsDraw;
      TeamResult[n].HRem++;
    } else {
      TeamResult[n].HPoints+=pointsLost;
      TeamResult[n].HAPoints+=pointsWon;
      if (prefs.pointsPenalty && penalty)
	TeamResult[n].HLostPen++;
      else
	TeamResult[n].HLost++;
    }
  }

  if (TeamResult[n].Goals > maxValues.goals) {
    maxValues.goals = TeamResult[n].Goals;
  }
  if (TeamResult[n].AGoals > maxValues.agoals) {
    maxValues.agoals = TeamResult[n].AGoals;
  }
  if (abs(TeamResult[n].Goals-TeamResult[n].AGoals) > maxValues.diff) {
    maxValues.diff = abs(TeamResult[n].Goals-TeamResult[n].AGoals);
  }
}

static Boolean EvalResult(matchType *m, Int8 matchday)
{
  if ( m->t1==0 || m->t2==0 || 
       m->t1>prefs.NumTeams || m->t2>prefs.NumTeams ) {
    DisplayErrorMessageAndStop(ID_ErrorMessages);
    return false;
  }
  
  if (m->r1 != 0xff && m->r2 != 0xff) {
    if (stats.predicate(m,m->t1,matchday)) {
      AddTeamResult(m->t1,m->r1,m->r2,m->penalty,true);
    }
    if (stats.predicate(m,m->t2,matchday)) {
      AddTeamResult(m->t2,m->r2,m->r1,m->penalty,false);
    }
  }
  return true;
}

static Boolean EvalHandicap(handicapType *h, Int8 i)
{
  if (h->type == pointHandicap) {
    TeamResult[h->team-1].Points += h->val;
  } else {
    TeamResult[h->team-1].Goals += h->val;
  }
  return true;
}

/*
 * Loop over all match days specified in "stats",
 * apply match and handicap callback functions for 
 * every match and handicap.
 * So this function adds up all goals and points of
 * the league.
 */
Boolean LoopRecords(matchTypeCallback mCallback, 
		    handicapTypeCallback hCallback, UInt32 mask)
{
  Int16 i, j;
  Int8 numHandicaps, numTeams;
  char *p, *q;
  matchType *m;
  handicapType *h;

  for (i=0; i<=stats.last; i++) {

    p = q = MemHandleLock(DmQueryRecord(db,i+2));
    numTeams = (Int8)*p++;
    numHandicaps = (Int8)*p++;
    
    if (mCallback) {
      for (j=0; j<numTeams; j++) {
	m = (matchType*)p;

	// first check mask, that team is involved
	if ( (mask & (1UL<<(m->t1-1))) && 
	     (mask & (1UL<<(m->t2-1))) ) {
	
	  if (!mCallback(m, i)) {
	    MemPtrUnlock(q);
	    prefs.Version = 0; // don't use this file again...
	    return false;
	  }

	}
	
	p += sizeof(matchType);
      }
    } else {
      p += numTeams*sizeof(matchType);
    }

    if (stats.evalHandicap && hCallback) {
      for (j=0; j<numHandicaps; j++) {
	h = (handicapType*)p;
	// check if team is involved
	if ( (mask & (1UL<<(h->team-1))) )
	  hCallback(h, i);
	p += sizeof(handicapType);
      }
    }

    MemPtrUnlock(q);
  }

  return true;
}

/*
 * compute the positions where in the table the ranking changes
 */
static UInt32 computePartition(void)
{
  Int8 prevPosition = -1;
  UInt32 partition = 0, i;

  SysInsertionSort(TeamResult,
		   prefs.NumTeams,
		   sizeof(TeamResultType),
		   cmpByPosition,
		   0);
  for (i=0; i<prefs.NumTeams; i++) {
    if (TeamResult[i].Position != prevPosition) {
      partition = partition | (1UL<<i);
      prevPosition = TeamResult[i].Position;
    }
  }
  return partition;
}

/*
 * compute the table of a league according to the "stats"
 * settings.
 */
Boolean ComputeTable(UInt32 teamMask)
{
  Int16 i,j,t;
  UInt32 partition;
  Int16 partitionStartPoint, partitionEndPoint;
  Boolean res;

  if (teamMask == 0xffffffff)
    TeamResult = MemHandleLock(TeamResultHandle);

  MemSet(TeamResult,prefs.NumTeams*sizeof(TeamResultType),0);

  // first step: reset previous values to zero.
  for (i=0; i<prefs.NumTeams; i++) {
    TeamResult[i].teamNo = i+1;
  }

  // second step: loop over all records necessary and sum up
  // goals, points and handicaps.
  if ( (res = LoopRecords(EvalResult,EvalHandicap,teamMask)) ) {

    partition = 1; // initial partition of table

    // we do table/partition sorting for each tiebreaker
    for (t=0; t<8; t++) {
      if (prefs.tiebreaker[t] != tieBreakerNone) {

	// compute start and end position of a partition;
	// initial partition is the whole table.
	partitionStartPoint = 0;
	for ( partitionEndPoint = partitionStartPoint+1; 
	      partitionEndPoint <= prefs.NumTeams; 
	      partitionEndPoint++ ) {
	  if ( (partition & (1UL<<partitionEndPoint)) || 
	       partitionEndPoint == prefs.NumTeams) {

	    // now sort the partition by selected tiebreaker.

	    // handle "direct encounter" differently, because a separate
	    // "sub table" has to be built with all teams involved.
	    if (prefs.tiebreaker[t] == tieBreakerDirectCompare 
		&& partitionEndPoint-partitionStartPoint > 1) { // direct comparison 
	      
	      UInt32 mask = 0;

	      // if we have recursively come to this point, there is
	      // not much to check. Procede with next tiebreaker.
	      if (teamMask != 0xffffffff) 
		continue;

	      // build mask of involved teams
	      for ( i=partitionStartPoint; i<partitionEndPoint; i++) {
		mask = mask | (1UL<< (TeamResult[i].teamNo-1));
	      }

	      // save TeamResult table for later reference
	      BackupTeamResult = MemHandleLock(BackupTeamResultHandle);
	      MemMove(BackupTeamResult, TeamResult, prefs.NumTeams*sizeof(TeamResultType));

	      // compute sub table, evaluate results
	      ComputeTable(mask);
	      for ( i=partitionStartPoint; i<partitionEndPoint; i++) {
		for (j=0; j<prefs.NumTeams; j++) {
		  if (BackupTeamResult[i].teamNo == TeamResult[j].teamNo)
		    BackupTeamResult[i].Position += TeamResult[j].Position;
		}
	      }

	      // restore original "corrected" TeamResult.
	      MemMove(TeamResult, BackupTeamResult, prefs.NumTeams*sizeof(TeamResultType));
	      MemHandleUnlock(BackupTeamResultHandle);

	    } else {
	      char sort;
	      for ( i=partitionStartPoint; i<partitionEndPoint-1; i++) {
		if ( (teamMask & (1UL<<(TeamResult[i].teamNo-1))) ) {
		  for (j=i+1; j<partitionEndPoint; j++) {
		    if ( (teamMask & (1UL<<(TeamResult[j].teamNo-1))) ) { 
		      sort = cmpTeamsByTiebreaker(&TeamResult[i],
						  &TeamResult[j],
						  prefs.tiebreaker[t]);
		      if (sort<0) TeamResult[i].Position++;
		      if (sort>0) TeamResult[j].Position++;
		    }
		  }
		}
	      }
	    }

	    partitionStartPoint = partitionEndPoint;
	  }
	}

	partition = computePartition();
      }
    }

  }
  if (teamMask == 0xffffffff)
    MemHandleUnlock(TeamResultHandle);

  return res;
}

/**
 * Form Compute Table Functions
 */

static void DrawTableItem(void* tableP, Int16 row, Int16 column, 
                            RectangleType *bounds)
{
  Int16 n, i, offset, toff;
  UInt16 x;
  Char buf[25]; // "buf" holds the string to be drawn
  IndexedColorType color = 255, take;

  n = TblGetItemInt(tableP, row, column);
  x = bounds->topLeft.x;

  toff = landscape?0:tableView;

  if (row == 0) { // table head

    FntSetFont(boldFont);
    offset = column;
    if (column>1)
      offset = prefs.tableSettings[(toff+column-2)%10]+2;
    StrCopy(buf,GetString(ID_tSTR_TableHead+offset));

  } else { // table body

    FntSetFont(stdFont);
    // use background colors for devices that can handle them
    // to draw special table positions.
    if (FeatureSet35) {
      for (i=0; i<5; i++) {
	if ( (prefs.colors[i].color != 255 || appPrefs.color[i] != 255)
	     && prefs.colors[i].from <= row 
	     && prefs.colors[i].to >= row ) {
	  take = prefs.colors[i].color;
	  if (take == 255) 
	    take = appPrefs.color[i];
	  color = WinSetBackColor(take);
	  bounds->extent.x ++;
	  WinEraseRectangle(bounds,0);
	  bounds->extent.x --;
	}
      } 
    }

    if (column == 0) {
      StrIToA(buf,n);
      StrCat(buf,".");
      x += bounds->extent.x - FntCharsWidth(buf,StrLen(buf)) - 1;
      
    } else if (column == 1) {
      // Draw team name, and if fav team, also underline it
      TeamResult = MemHandleLock(TeamResultHandle);
      StrCopy(buf, theTeams[TeamResult[n].teamNo-1]);
      if (TeamResult[n].teamNo == prefs.favTeam)
	WinSetUnderlineMode(solidUnderline);
      MemHandleUnlock(TeamResultHandle);
      
    } else {
      
      TeamResult = MemHandleLock(TeamResultHandle);
      switch (prefs.tableSettings[(toff+column-2)%10]) {
	
      case tableSettingsMatches:
	StrIToA(buf,TeamResult[n].Matches);
	break;
      case tableSettingsGoals:
	StrIToA(buf,TeamResult[n].Goals);
	break;
      case tableSettingsGoalsAgainst:
	buf[0] = ':';
	StrIToA(buf+1,TeamResult[n].AGoals);
	break;
      case tableSettingsGoalsDiff:
	buf[0] = ' ';
	offset = TeamResult[n].Goals-TeamResult[n].AGoals;
        if (offset < 0) buf[0] = '-';
        if (offset > 0) buf[0] = '+';
	StrIToA(buf+1,abs(offset));
	break;
      case tableSettingsPts:
	StrIToA(buf,TeamResult[n].Points);
	break;
      case tableSettingsPtsAgainst:
	buf[0] = ':';
	StrIToA(buf+1,TeamResult[n].APoints);
	break;
      case tableSettingsWon:
	StrIToA(buf,TeamResult[n].Won);
	break;
      case tableSettingsWonPen:
	StrIToA(buf,TeamResult[n].WonPen);
	break;
      case tableSettingsDraw:
	StrIToA(buf,TeamResult[n].Rem);
	break;
      case tableSettingsLostPen:
	StrIToA(buf,TeamResult[n].LostPen);
	break;
      case tableSettingsLost:
	StrIToA(buf,TeamResult[n].Lost);
	break;
      }
      MemHandleUnlock(TeamResultHandle);
      
    }

  }

  if (column>1) {
    if (prefs.tableSettings[(toff+column-2)%10] != tableSettingsGoalsAgainst 
	&& prefs.tableSettings[(toff+column-2)%10] != tableSettingsPtsAgainst) {
      x += bounds->extent.x - FntCharsWidth(buf,StrLen(buf));
    }
  }

  WinSetClip(bounds);
  WinDrawChars(buf, StrLen(buf), x, bounds->topLeft.y);
  WinResetClip();
  FntSetFont(stdFont);
  WinSetUnderlineMode(noUnderline);
  if (color != 255 && FeatureSet35) {
      WinSetBackColor(color);
  }

  // use lines to draw special table positions on devices
  // that can't handle background colors.
  if (!FeatureSet35 && row>1) {
    for (i=0; i<5; i++) {
      if ( prefs.colors[i].to == row ) {
	WinDrawLine(bounds->topLeft.x,
		    bounds->topLeft.y+bounds->extent.y-1,
		    bounds->topLeft.x+bounds->extent.x,
		    bounds->topLeft.y+bounds->extent.y-1);
      }
      if ( prefs.colors[i].from == row ) {
	WinDrawLine(bounds->topLeft.x,
		    bounds->topLeft.y-1,
		    bounds->topLeft.x+bounds->extent.x,
		    bounds->topLeft.y-1);
      }
    }
  }
}  

/*
 * initialize table variables to default values;
 */
static void InitFormTable(void)
{
  Int16 i, j;
  leagueTable = Id2Ptr(ID_FrmTableTable);

  TblSetRowHeight(leagueTable, 0, 15);
  for (i=0; i<=prefs.NumTeams; i++) {
    TblSetRowSelectable(leagueTable, i, i>0?true:false);
    for (j=0; j<11; j++) {
      TblSetItemStyle(leagueTable, i, j, customTableItem);
      TblSetColumnUsable(leagueTable, j, j<7?true:false);
      TblSetCustomDrawProcedure(leagueTable, j, DrawTableItem);
    }
  }
  for (i=1; i<TblGetNumberOfRows(leagueTable); i++) {
    TblSetRowUsable(leagueTable, i, false);
  }
}

static void RedrawFormTable(FormType* frm)
{
  FrmDrawForm(frm);
  DrawButton(ID_FrmTableButton+0,ID_Bitmap+2);
  DrawButton(ID_FrmTableButton+1,ID_Bitmap+5);
  DrawButton(ID_FrmTableButton+2,ID_Bitmap+6);
  DrawButton(ID_FrmTableButton+3,ID_Bitmap+8);
}

/*
 * fills in the values for the table and redraws it.
 */
static void InitTable(FormType* frm)
{
  Int16 n, i, x, y;

  TeamResult = MemHandleLock(TeamResultHandle);
  for (n=0; n<prefs.NumTeams; n++) {
    TblSetItemInt(leagueTable, n+1, 0, TeamResult[n].Position+1);
    for (i=1; i<11; i++) {
      TblSetItemInt(leagueTable, n+1, i, n);
    }
  }
  MemHandleUnlock(TeamResultHandle);

  WinGetDisplayExtent( &x, &y );

  // Landscape Mode
  if (x > y) {
    tableSize = 10;
    for (i=7; i<11; i++) {
      TblSetColumnUsable(leagueTable, i, true);
    }
    landscape = true;
  }

  // Portrait Mode
  if (x <= y) {
    tableSize = 16;
    for (i=7; i<11; i++) {
      TblSetColumnUsable(leagueTable, i, false);
    }
    landscape = false;
  }

  // Square Mode
  if (x == y)
    tableSize = 10;

  RedrawFormTable(frm);
  toggleMatchdayLabel(false,ID_FrmTableButton+6);
}

/*
 * compute width of requested string in bold font
 */
Int16 GetWidthOfString(Int16 id)
{
  Int16 wLabel;
  char * lStr = GetString(id);
  FontID font = FntGetFont();
  FntSetFont(boldFont);
  wLabel = FntCharsWidth(lStr,StrLen(lStr));
  FntSetFont(font);
  return wLabel;
}

/*
 * dynamically adjust table columns to fit widths
 */
static void ResizeTableColumns()
{
  Int16 i, n=56, min, max, width, wLabel;
  char buf[7];
 
  if (landscape) {
    min = 0;
    max = 9;
  } else {
    min = tableView;
    max = tableView+5;
  }
  for (i=min; i<max; i++) {
    width = 15;
    switch (prefs.tableSettings[i]) {
    case tableSettingsGoals:
      StrIToA(buf,maxValues.goals);
      width = FntCharsWidth(buf,StrLen(buf))+1;
      wLabel = GetWidthOfString(ID_tSTR_TableHead+3);
      if ( wLabel > width )
	width = wLabel;
      break;
    case tableSettingsGoalsAgainst:
      StrIToA(buf,maxValues.agoals);
      width = FntCharsWidth(buf,StrLen(buf))+1;
      wLabel = GetWidthOfString(ID_tSTR_TableHead+4);
      if ( wLabel > width )
	width = wLabel;
      break;
    case tableSettingsGoalsDiff:
      StrIToA(buf,maxValues.diff);
      StrCat(buf," +");
      width = FntCharsWidth(buf,StrLen(buf));
      wLabel = GetWidthOfString(ID_tSTR_TableHead+5);
      if ( wLabel > width )
	width = wLabel;
      break;
    }
    TblSetColumnWidth(leagueTable, i+2-min, width);
    n -= width-15;
  }
  TblSetColumnWidth(leagueTable, 1, n);

}

Boolean SetTableFilter(Int16 selection)
{
  Boolean handled = false;
  Int16 first = 0, last = prefs.NumMatchDays - 1;

  switch ( selection ) {
  case 0:
    handled = true; 
    break;
  case 1: // table for day "matchDay"
    last = matchDay-1;
    handled = true; 
    break;
  case 2: // second round matches only
    first = prefs.NumMatchDays>>1;
    handled = true; 
    break;
  case 3: // last 5 match days
    first = matchDay - 5;
    if (first<0) first = 0;
    last = matchDay-1;
    handled = true; 
    break;
  case 4: // home matches
    stats.predicate = isHomeMatch;
    handled = true; 
    break;
  case 5: // away matches
    stats.predicate = isAwayMatch;
    handled = true; 
    break;
  }
  if (handled) {
    stats.first = first;
    stats.last  = last;
    stats.evalHandicap = selection?false:true;
    if (selection<4)
      stats.predicate = inBetween;
  }
  return handled;
}

Boolean TableFormHandleEvent(EventPtr e)
{
  Boolean handled, flag;
  static Boolean needToRedraw;
  static ScrollBarPtr scrollP;
  ListPtr pLst;
  static FormType* frm;

  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {

  case frmOpenEvent:
    needToRedraw = false;
    frm = FrmGetActiveForm();
    tableSize = 10;
    SetTableFilter(lastSelection);
    DateOfMatchday = GetMostUsedDate();
    InitFormTable();
    scrollP = Id2Ptr(ID_FrmTableScroll);
    maxValues.goals = maxValues.agoals = maxValues.diff = 99;
    if (ComputeTable(0xffffffff)) {
      InitTable(frm);
      ResizeTableColumns();
      RedrawTable(leagueTable, scrollP, TableRememberPosFirst, 
		  prefs.NumTeams+1, 1, tableSize);
    }
    pLst = Id2Ptr(ID_FrmTableFilter+1);
    LstSetSelection(pLst,lastSelection);
    CtlSetLabel(Id2Ptr(ID_FrmTableFilter), 
		   LstGetSelectionText(pLst,lastSelection));
    ReturnToForm = ID_FrmTable;
    handled  = true; 
    break;

  case frmUpdateEvent:
    InitTable(frm);
    ResizeTableColumns();
    RedrawTable(leagueTable, scrollP, TableRememberPosFirst, 
		prefs.NumTeams+1, 1, tableSize);
    handled = true;
    break;

  // work around for bitmap buttons not being redrawn
  // after showing a popup (like FrmHelp())
  case winEnterEvent:
    if (needToRedraw) {
      needToRedraw = false;
      RedrawFormTable(frm);
    }
    break;

  case menuEvent:
    switch(e->data.menu.itemID) {
    // Help
    case ID_TableMenuItem:
      MyDoDialog(ID_FrmHelp, ID_FrmTable, "Tipps", FrmHelpHandleEvent, InitFrmHelp);
      needToRedraw = true;
      break;

    // About Bundesliga
    case ID_TableMenuItem+1:
      FrmPopupForm(ID_FrmAbout);
      break;

    // Table Settings
    case ID_TableMenuItem+2:
      FrmGotoForm(ID_FrmTableSettings);
      break;
    }
    handled = true;
    break;

  // Scrollbar Event
  case sclRepeatEvent:
    TableRememberPosFirst = e->data.sclRepeat.newValue+1;
    RedrawTable(leagueTable, scrollP, TableRememberPosFirst, 
		prefs.NumTeams+1, 1, tableSize);
    break;
  
  case ctlSelectEvent:
    switch (e->data.ctlSelect.controlID) {

    // return to results/fixture form
    case ID_FrmTableButton:
      FrmGotoForm(ID_FrmBl);
      handled = true; 
      break;
    
    // fixture form
    case ID_FrmTableButton+2:
      FrmGotoForm(ID_FrmShowFixture);
      handled = true; 
      break;
    
    // Matchday/Date label upper right corner
    case ID_FrmTableButton+6:  
      toggleMatchdayLabel(true,ID_FrmTableButton+6);
      handled = true; 
      break;

    case ID_FrmTableButton+4: /* prev match day button */
      if (matchDay>1) {
        --matchDay;
	FrmGotoForm(ID_FrmTable);
      }
      handled = true; 
      break;
    
    case ID_FrmTableButton+5: /* next match day button */
      if (matchDay < prefs.NumMatchDays) {
        ++matchDay;
	FrmGotoForm(ID_FrmTable);
      }
      handled = true; 
      break;

    case ID_FrmTableButton+3: /* jump to button */
      FrmGotoForm(ID_FrmGoto);
      handled = true; 
      break;

    }
    break;

  case popSelectEvent:
    // different table views (home/away/...) button
    if (e->data.popSelect.selection != e->data.popSelect.priorSelection) {
      lastSelection = e->data.popSelect.selection;
      handled = SetTableFilter(lastSelection);
      FrmGotoForm(ID_FrmTable);
    }
    break;

  case keyDownEvent:
    flag = false;
    if (e->data.keyDown.chr == vchrPageUp) {
      if (TableRememberPosFirst > 1) {
        flag = true;
	TableRememberPosFirst -= 10;
	if (TableRememberPosFirst < 1) 
	  TableRememberPosFirst = 1;
      }
      handled = true; 
    }
    if (e->data.keyDown.chr == vchrPageDown) {
      if (TableRememberPosFirst+10 <= prefs.NumTeams) {
        flag = true;
	TableRememberPosFirst += 10;
	if (TableRememberPosFirst+10 >= prefs.NumTeams)
	  TableRememberPosFirst = prefs.NumTeams - 9;
      }
      handled = true; 
    }

    if (e->data.keyDown.chr == vchrRockerLeft || 
	e->data.keyDown.keyCode == fiveWayLeft) {
      FrmGotoForm(ID_FrmBl);
      handled = true; 
    }

    if (e->data.keyDown.chr == vchrRockerRight || 
	e->data.keyDown.keyCode == fiveWayRight) {
      FrmGotoForm(ID_FrmShowFixture);
      handled = true; 
    }

    if (e->data.keyDown.chr == vchrRockerCenter || 
        (e->data.keyDown.keyCode == fiveWaySelect &&
         (e->data.keyDown.modifiers & autoRepeatKeyMask) == 0) ) {
      EventType evt;
      evt.eType = ctlSelectEvent;
      evt.data.ctlSelect.controlID = ID_FrmTableFilter;
      EvtAddEventToQueue(&evt);
      handled = true; 
    }

    if (flag)
      RedrawTable(leagueTable, scrollP, TableRememberPosFirst, 
		  prefs.NumTeams+1, 1, tableSize);
    break;

  // tapped on a team name => show statistics
  case tblSelectEvent:
    TblUnhighlightSelection(leagueTable);
    if ( e->data.tblSelect.column == 1 ) {
      TeamResult = MemHandleLock(TeamResultHandle);
      stats.team = TeamResult[e->data.tblSelect.row-1].teamNo;
      MemHandleUnlock(TeamResultHandle);
      FrmGotoForm(ID_FrmStatistics);
    } else {
      tableView = 5-tableView;
      ResizeTableColumns();
      RedrawTable(leagueTable, scrollP, TableRememberPosFirst, 
		  prefs.NumTeams+1, 1, tableSize);
    }
    handled = true;
    break;

  default:
    break;
  }

  return handled;
}

Boolean TableSettingsFormHandleEvent(EventPtr e) 
{
  Boolean handled;
  ListPtr pLst;
  static FormPtr frm;
  Int16 i;

  if ( (handled = ResizeHandleEvent(e)) ) {
    return true;
  }

  switch (e->eType) {
  case frmOpenEvent:
    frm = FrmGetActiveForm();
    FrmDrawForm(frm);
    for (i=0; i<10; i++) {
      pLst = Id2Ptr(ID_FrmTableSettings+2*i+11);
      LstSetSelection(pLst, prefs.tableSettings[i]);
      CtlSetLabel(Id2Ptr(ID_FrmTableSettings+2*i+10),
		  LstGetSelectionText(pLst,LstGetSelection(pLst)));
    }
    handled = true;
    break;

  case ctlSelectEvent:
    switch (e->data.ctlEnter.controlID) {

    // Ready Button
    case ID_FrmTableSettings:
      for (i=0; i<10; i++) {
	pLst = Id2Ptr(ID_FrmTableSettings+2*i+11);
	prefs.tableSettings[i] = LstGetSelection(pLst);
      }
    
    // Cancel Button
    case ID_FrmTableSettings+1:
      FrmGotoForm(ID_FrmTable);
      handled = true; 
      break;
    
    }
    break;

  case penDownEvent:
    // this is the position of the (i)
    if (e->screenX >= 144 && e->screenX <= 155 &&
	e->screenY >= 0 && e->screenY <= 11) {
      // this way we don't have to use the default bold font.
      MyDoDialog(ID_FrmHelp, ID_FrmTableSettings, "Tipps", FrmHelpHandleEvent, InitFrmHelp);
      handled =true;
    }

  default:
    break;
  }

  return handled;
}
